package provide DS9 1.0

proc UpdateAnalysisMenu {} {
    global ds9
    global current
    global analysisMenu
    global analysisType

    if {$current(frame) != ""} {
	.menuBar entryconfig Analysis -state normal
	for {set i 0} {$i<$ds9(analysisItem)} {incr i} {
	    if {[$current(frame) has fits]} {
		set fn [$current(frame) get fits file name number 1]
		if {[regexp "(.*)\.$analysisType($i)(.*)" $fn matched] } {
		    .menuBar.analysis entryconfig $analysisMenu($i) \
			-state normal
		} else {
		    .menuBar.analysis entryconfig $analysisMenu($i) \
			-state disabled
		}
	    } else {
		.menuBar.analysis entryconfig $analysisMenu($i) \
		    -state disabled
	    }
	}
    } else {
	.menuBar entryconfig Analysis -state disabled
    }
}

proc InitAnalysisFile {} {
    global ds9

    # look for local version
    ProcessAnalysisFile "./$ds9(extAnalysis)"

    # look for $HOME version
    ProcessAnalysisFile "~/$ds9(extAnalysis)"

    UpdateAnalysisMenu
}

proc ProcessAnalysisFile {fn} {
    global ds9
    global analysisMenu
    global analysisType
    global analysisCmd

    set status 1
    if [file exists "$fn"] {
	set id [open $fn r]
	while {[gets $id line] >= 0} {
	    # empty line
	    if {[string length $line] == 0} continue
	    # comments	    
	    if {[string range $line 0 0] == "\#"} continue
	    # else
	    switch $status {
		1 {
		    set menu "$line"
		    set template ""
		    set status 2
		}
		2 {
		    set template "$line"
		    set status 3
		}
		3 {
		    # eat the line
		    set status 4
		}
		4 {
		    if {"$menu" != "" && "$template" != "" && "$line" != ""} {
			foreach t $template {
			    set f $ds9(analysisItem)
			    set analysisMenu($f) "$menu"
			    set analysisType($f) "$t"
			    set analysisCmd($f) "$line"
			    .menuBar.analysis add command -label "$menu" \
				-command "AnalysisCmd $f"
			    incr ds9(analysisItem)
			}
		    }

		    set status 1
		}
	    }
	}
	close $id
    }
}

proc AnalysisCmd {which} {
    global analysisType
    global analysisCmd
    global current

    set str $analysisCmd($which)
    if {$current(frame) != ""} {
	if {[$current(frame) has fits]} {
	    set fn [$current(frame) get fits full file name number 1]
	    regsub -all {\$filename} $str $fn result
	    set str $result

	    set str [MacroRegion $str {\$include_regions_pixels} \
      "$current(frame) marker list pros image include yes semicolon"]
	    set str [MacroRegion $str {\$include_regions_degrees} \
      "$current(frame) marker list pros fk5 degrees include yes semicolon"]
	    set str [MacroRegion $str {\$include_regions_hms} \
      "$current(frame) marker list pros fk5 sexagesimal include yes semicolon"]
	    set str [MacroRegion $str {\$include_regions} \
      "$current(frame) marker list pros image include yes semicolon"]

	    set str [MacroRegion $str {\$exclude_regions_pixels} \
      "$current(frame) marker list pros image include no semicolon"]
	    set str [MacroRegion $str {\$exclude_regions_degrees} \
      "$current(frame) marker list pros fk5 degrees include no semicolon"]
	    set str [MacroRegion $str {\$exclude_regions_hms} \
      "$current(frame) marker list pros fk5 sexagesimal include no semicolon"]
	    set str [MacroRegion $str {\$exclude_regions} \
      "$current(frame) marker list pros image include no semicolon"]

	    set str [MacroRegion $str {\$regions_pixels} \
      "$current(frame) marker list pros image semicolon"]
	    set str [MacroRegion $str {\$regions_degrees} \
      "$current(frame) marker list pros fk5 degrees semicolon"]
	    set str [MacroRegion $str {\$regions_hms} \
      "$current(frame) marker list pros fk5 sexagesimal semicolon"]
	    set str [MacroRegion $str {\$regions} \
      "$current(frame) marker list pros image semicolon"]
	}
    }
    
    if [catch {set ch [open "| $str"]}] {
	Error "An error as occured invoking the Analysis task."
	return
    }
    DisplayAnalysisDialog [read -nonewline $ch]
    close $ch
}

proc MacroRegion {str exp cmd } {
    # get any regions
    set region [eval $cmd]

    # we have to quote {"}, else problems down the road
    regsub -all {\"} $region {\\"} result
    set region $result

    # if we have nothing, set to none
    if {$region == ""} {
	set region "NONE"
    }

    regsub -all $exp $str "$region " result
    set str $result

    return $str
}

proc DisplayAnalysisDialog {txt} {
    global alDialog

    set alDialog(top) .al
    set alDialog(mb) .alMB
    set alDialog(txt) "$txt"

    SimpleTextDialog alDialog 80 20 "Analysis" insert bottom
}
