package provide DS9 1.0

# capture general errors

proc tkerror {err} {
    global cursor

    tk_messageBox -message "An internal error has been detected: $err" \
	-type ok -icon error

    # reset cursor

    set cursor(count) 0
    UnsetWatchCursor
}

# capture xpa/interactive errors

proc Error {message} {
    global ds9

    if {$ds9(xpa) == 0} {
	tk_messageBox -message $message -type ok -icon error
    } else {
	xpaerror $ds9(xpa) $message
    }
}

proc Sex2D {str} {
    scan $str "%d:%d:%f" h m s
    return [expr $h+($m/60.)+($s/60./60.)]
}

proc SetCursor {cursor} {
    global ds9
    global iis
    global current
    global canvas

    # if init phase, don't change cursor
    if {$ds9(init)} {
	return
    }

    # if iis cursor mode, don't change cursor
    if {$iis(state)} {
	return
    }

    if {$ds9(cursor) && ($current(cursor) != $cursor)} {
	set current(cursor) $cursor

	if {$cursor != ""} {
	    $canvas(name) configure -cursor $cursor
	} else {
	    $canvas(name) configure -cursor {}
	}
    }
}

# NOTE
# This is where everything gets done. In particular, <Configure> events are
# processed here. Also, any screen updates are done. In short, if it changes
# the geometry or changes the contents, you need to call Set/UnsetWatchCursor

proc SetWatchCursor {} {
    global ds9
    global cursor
    global canvas

    global debug
    if {$debug(watch)} {
	puts "SetWatchCursor Start $cursor(count)"
    }

    # if init phase, don't change cursor

    if {$ds9(init)} {
	return
    }

    if {$cursor(count) == 0} {
	set cursor(save) [$canvas(name) cget -cursor]
	$canvas(name) configure -cursor {}
	.ds9 configure -cursor watch
	update idletasks
    }
    incr cursor(count)

    if {$debug(watch)} {
	puts "SetWatchCursor End $cursor(count)"
    }
}

proc UnsetWatchCursor {} {
    global ds9
    global cursor
    global canvas

    global debug
    if {$debug(watch)} {
	puts "UnsetWatchCursor Start $cursor(count)"
    }

    # if init phase, don't change cursor

    if {$ds9(init)} {
	return
    }

    if {$cursor(count)>0} {
	incr cursor(count) -1
    }

    if {$cursor(count) == 0} {
	.ds9 configure -cursor {}
	$canvas(name) configure -cursor $cursor(save)
	update idletasks
    }    

    if {$debug(watch)} {
	puts "UnsetWatchCursor End $cursor(count)"
    }
}

proc OpenFileDialog {parent prevName} {
    upvar $prevName prev

    global ds9
    global tk_strictMotif

    set result ""
    switch $ds9(dialog) {
	motif {
	    # Theres a bug with strictmotif. 
	    # If tk_strictMotif is set to 1, buttons are
	    # rendered smaller than if 0.
	    set tk_strictMotif 1
	    set result [tk_getOpenFile -parent $parent -initialfile "$prev"]
	    set tk_strictMotif 0
	}
	windows {
	    set result [tk_getOpenFile -parent $parent -initialfile "$prev"]
	}
	blueplate {
	    set result [tk_getOpenFile -blueplate 1 -parent $parent \
			    -initialfile "$prev"]
	}
    }
    set prev $result
    return $result
}

proc SaveFileDialog {parent prevName} {
    upvar $prevName prev

    global ds9
    global tk_strictMotif

    set result ""
    switch $ds9(dialog) {
	motif {
	    set tk_strictMotif 1
	    set result [tk_getSaveFile -parent $parent -initialfile "$prev"]
	    set tk_strictMotif 0
	}
	windows {
	    set result [tk_getSaveFile -parent $parent -initialfile "$prev"]
	}
	blueplate {
	    set result [tk_getSaveFile -blueplate 1 -parent $parent \
			    -initialfile "$prev"]
	}
    }
    set prev $result
    return $result
}

proc AboutBox {} {
    global about

    set w $about(top)
    set width 340
    set height 300

    DialogCreate $w $about(title) -borderwidth 2
    frame $w.c -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.c $w.buttons -fill x -ipadx 4 -ipady 4

    canvas $w.c.c -background white -height $height -width $width
    pack $w.c.c
    
    set about(sun) [image create photo -format gif -file sun.gif]
    set about(tk) [image create photo -format gif \
		       -file ../lib/tk8.3/images/logo100.gif]

    $w.c.c create image 0 0 -image $about(sun) -anchor nw
    $w.c.c create image 60 270 -image $about(tk) -anchor s
    $w.c.c create text 140 20 -text $about(message) -anchor nw

    button $w.buttons.ok -text "OK" -default active \
	-command {set about(ok) 1}
    grid $w.buttons.ok -padx 10

    bind $w <Return> {set about(ok) 1}

    DialogCenter $w
    DialogWait $w about(ok)
    DialogDismiss $w

    if {$about(ok)} {
	image delete $about(sun)
	image delete $about(tk)
    }
}

proc QuitDS9 {} {
    IISClose
    focus {}
    exit
}

proc OpenSource {} {
    global ds9

    set fileName [OpenFileDialog $ds9(top) ds9(lastTclName)]
    if {$fileName != ""} {
	uplevel #0 source $fileName
    }
}

proc UpdateView {} {
    global debug
    if {$debug(layout)} {
	puts "UpdateView"
    }

    SetWatchCursor
    LayoutView
    UnsetWatchCursor
}

proc LayoutView {} {
    global current
    global panel
    global info
    global pan
    global mag
    global buttons
    global canvas
    global colorbar
    global graph
    global graphHorz
    global graphVert
    global ds9

    global debug
    if {$debug(layout)} {
	puts "LayoutView"
    }

    grid forget $panel(name)
    pack forget $info(name)
    pack forget $pan(name)
    pack forget $mag(name)
    grid forget $buttons(name)
    grid forget $canvas(name)
    grid forget $graph(spacer)
    grid forget $graphVert(name)
    grid forget $graphHorz(name)
    grid forget $colorbar(name)

    if {$info(view) || $mag(view) || $pan(view)} {
	grid $panel(name) -row 0 -sticky ew -columnspan 3
    }

    if {$info(view)} {
	pack $info(name) -side left -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    }

    if {$mag(view)} {
	pack $mag(name) -side right -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    }

    if {$pan(view)} {
	pack $pan(name) -side right -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    }

    if {$buttons(view)} {
	grid $buttons(name) -row 1 -sticky ew -columnspan 3
    }

    if {$canvas(auto)} {
	grid $canvas(name) -row 2 -sticky news
    } else {
	$canvas(name) configure -width $canvas(size) -height $canvas(size)
	grid $canvas(name) -row 2
    }

    if {$graphVert(view)} {
	switch $ds9(displaymode) {
	    single {grid $graphVert(name) -row 2 -column 2 -rowspan 2 \
			-sticky ns}
	    tile {}
	    blink {}
	}
    }

    if {$graphHorz(view) || $graphVert(view)} {
	switch $ds9(displaymode) {
	    single {grid $graph(spacer) -row 3 -column 1}
	    tile {}
	    blink {}
	}
    }

    if {$graphHorz(view)} {
	switch $ds9(displaymode) {
	    single {grid $graphHorz(name) -row 4 -columnspan 2 -sticky ew}
	    tile {}
	    blink {}
	}
    }

    if {$colorbar(view)} {
	grid $colorbar(name) -row 5 -sticky ew
    }
}

proc ToggleFreeze {} {
    global ds9

    if {$ds9(freeze)} {
	set ds9(freeze) 0
	UnFreezeCanvas
	UnFreezePanner
    } else {
	set ds9(freeze) 1
	FreezeCanvas
	FreezePanner
    }
}

proc ChangeMode {} {
    global ds9
    global canvas

    bind $canvas(name) <Button-1> {}
    bind $canvas(name) <B1-Motion> {}
    bind $canvas(name) <ButtonRelease-1> {}

    foreach f $ds9(frames) {
	$f crosshair off
    }
    UpdateMarkerMenu

    UnselectAllMarkers
    ClearGraphData
    ClearInfoBox
    ClearPixelTable

    switch $ds9(mode) {
	pointer {SetCursor {}}
	crosshair {
	    foreach f $ds9(frames) {
		$f crosshair on
	    }
	    UpdateGraphData
	    SetCursor crosshair
	}
	colorbar {
	    bind $canvas(name) <Button-1> {ButtonColormap %x %y}
	    bind $canvas(name) <B1-Motion> {MotionColormap %x %y}
	    bind $canvas(name) <ButtonRelease-1> {ReleaseColormap %x %y}
	    SetCursor center_ptr
	}
	zoom {SetCursor sizing}
	pan {SetCursor fleur}
	rotate {SetCursor exchange}
	examine {SetCursor target}
	illustrate {SetCursor arrow}
    }
}

proc ProcessHelp {} {
    global argc
    global argv

    if {$argc > 0 && ([lindex $argv 0] == "-?" || 
		      [lindex $argv 0] == "-help" ||
		      [lindex $argv 0] == "help" ||
		      [lindex $argv 0] == "-h")} {
	DumpHelp
	exit
    }
}

proc ProcessColorCommandLine {} {
    global argc
    global argv
    global colorbar
    global ds9

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -private {set colorbar(private) true}
	}
	incr i
    }
}

proc BadVisualError {} {
    global ds9

    tk_messageBox -parent $ds9(top) -icon info -type ok -message \
	"Sorry, DS9 requires a Pseudocolor8, Truecolor8, Truecolor16, Truecolor24 visual be available."
    exit
}

proc ProcessCommandLine {} {
    global argc
    global argv

    global ds9
    global current
    global crosshair
    global contour
    global colorbar
    global wcs
    global info
    global pan
    global mag
    global grid
    global graph
    global graphHorz
    global graphVert
    global bin
    global scale
    global minmax
    global user
    global iis
    global marker

    set fileCount 0
    set i 0

    # Note: help is processed previously
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -private {}
	    
	    -align {
		set wcs(align) 1
		AlignWCSFrame
	    }
	    -array {
		incr i
		set item [lindex $argv $i]

		LoadCmdLine fileCount

		LoadArray $item
		set ds9(lastArrayName) $item
	    }
	    -blink {
		set ds9(displaymode) blink
		DisplayMode
	    }
	    -bin {incr i;ProcessBin argv i}
	    -cmap {
		incr i
		set colorbar(map) [lindex $argv $i]
		ChangeColormap
	    }
	    -colorbar {
		set colorbar(view) 1
		UpdateView
	    }
	    -nocolorbar {
		set colorbar(view) 0
		UpdateView
	    }
	    -contour {set contour(view) 1; ContourUpdate}
	    -nocontour {set contour(view) 0; ContourUpdate}
	    -coords {
		set item [lindex $argv [expr $i+1]]
		set value [FromYesNo [lindex $argv [expr $i+2]]]
		switch -- $item {
		    wcs {set info(wcs) $value}
		    detector {set info(detector) $value}
		    amplifier {set info(amplifier) $value}
		    physical {set info(physical) $value}
		    image {set info(image) $value}
		    default {}
		}
		UpdateInfoPanel

		incr i 2
	    }
	    -dss {
		LoadCmdLine fileCount
		incr i
		DSSCommandLine [lindex $argv $i]
	    }
	    -exit {QuitDS9}
	    -quit {QuitDS9}
	    -grid {set grid(view) 1; GridUpdate}
	    -nogrid {set grid(view) 0; GridUpdate}
	    -fifo {
		incr i
		set fifo [lindex $argv $i]
		if {$fifo!="none"} {
		    set iis(ififo) ${fifo}i
		    set iis(ofifo) ${fifo}o
		} else {
		    set iis(ififo) none
		    set iis(ofifo) none
		}
	    }
	    -fifo_only {
		set iis(port) 0
		set iis(unix) none
	    }
	    -help {
		DumpHelp
		exit
	    }
	    -horzgraph {
		set graphHorz(view) 1
		UpdateView
	    }
	    -nohorzgraph {
		set graphHorz(view) 0
		UpdateView
	    }
	    -info {
		set info(view) 1
		UpdateView
	    }
	    -noinfo {
		set info(view) 0
		UpdateView
	    }
	    -iconify {wm iconify $ds9(top)}
	    -invert {
		set colorbar(invert) 1
		InvertColorbar
	    }
	    -linear {
		set current(scale) linear
		ChangeColorScale
	    }
	    -log {
		set current(scale) log
		ChangeColorScale
	    }
	    -lower {lower $ds9(top)}
	    -magnifier {
		set mag(view) 1
		UpdateView
	    }
	    -nomagnifier {
		set mag(view) 0
		UpdateView
	    }
	    -minmax {
		set scale(mode) minmax
		ScaleMode
	    }
	    -mode {
		incr i
		set ds9(mode) [lindex $argv $i]
		ChangeMode
	    }
	    -mosaicimage {
		incr i
		set item [lindex $argv $i]

		LoadCmdLine fileCount

		LoadMosaicImageFile $item
		if {[regexp {http//.*} $item] || [regexp {ftp//.*} $item]} {
		    set ds9(lastURL) $item
		} else {
		    set ds9(lastMosaicName) $item
		}
	    }
	    -mosaicimages {
		incr i
		set item [lindex $argv $i]

		LoadCmdLine fileCount

		LoadMosaicFiles [lindex $argv $i]
		if {[regexp {http//.*} $item] || [regexp {ftp//.*} $item]} {
		    set ds9(lastURL) $item
		} else {
		    set ds9(lastMosaicName) $item
		}
	    }
	    -mosaic {
		incr i
		set item [lindex $argv $i]
		LoadFits $item mosaic
		if {[regexp {http//.*} $item] || [regexp {ftp//.*} $item]} {
		    set ds9(lastURL) $item
		} else {
		    set ds9(lastMosaicName) $item
		}
	    }
	    -orient {
		incr i
		set current(orient) [lindex $argv $i]
		ChangeOrient
	    }
	    -panner {
		set pan(view) 1
		UpdateView
	    }
	    -nopanner {
		set pan(view) 0
		UpdateView
	    }
	    -port {
		incr i
		set iis(port) [lindex $argv $i]
	    }
	    -inet_only -
	    -port_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(unix) none
	    }
	    -print {PostScript}
	    -raise {raise $ds9(top)}
	    -rass {
		LoadCmdLine fileCount
		incr i
		RASSCommandLine [lindex $argv $i]
	    }
	    -regionfile {
		incr i
		$current(frame) marker load $marker(format) [lindex $argv $i]
		set ds9(lastMarkerName) [lindex $argv $i]
	    }
	    -rotate {
		incr i
		set current(rotate) [lindex $argv $i]
		ChangeRotate
	    }
	    -scale -
	    -ztrans {
		incr i
		set current(scale) [lindex $argv $i]
		ChangeColorScale
	    }
	    -scalelimits {
		incr i
		set user(low) [lindex $argv $i]
		incr i
		set user(high) [lindex $argv $i]
		ScaleLimit $user(low) $user(high)
	    }
	    -scalemode {
		incr i
		set scale(mode) [lindex $argv $i]
		ScaleMode
	    }
	    -shm {

		incr i
		switch -- [lindex $argv $i] {
		    array {
			LoadCmdLine fileCount
			LoadShared array {} \
			    [lindex $argv [expr $i+1]] \
			    [lindex $argv [expr $i+2]] \
			    [lindex $argv [expr $i+3]] \
			    [lindex $argv [expr $i+4]]
			incr i 4
		    }
		    mosaic {
			LoadShared fits mosaic \
			    [lindex $argv [expr $i+1]] \
			    [lindex $argv [expr $i+2]] \
			    [lindex $argv [expr $i+3]] \
			    [lindex $argv [expr $i+4]]
			incr i 4
		    }
		    mosaicimage {
			LoadCmdLine fileCount
			LoadShared fits {mosaic image} \
			    [lindex $argv [expr $i+1]] \
			    [lindex $argv [expr $i+2]] \
			    [lindex $argv [expr $i+3]] \
			    [lindex $argv [expr $i+4]]
			incr i 4
		    }
		    key -
		    shmid {
			LoadCmdLine fileCount
			LoadShared fits {} \
			    [lindex $argv $i] \
			    [lindex $argv [expr $i+1]] \
			    [lindex $argv [expr $i+2]] \
			    [lindex $argv [expr $i+3]]
			incr i 3
		    }
		    default {
			LoadCmdLine fileCount
			LoadShared fits {} key \
			    [lindex $argv $i] \
			    [lindex $argv [expr $i+1]] \
			    [lindex $argv [expr $i+2]]
			incr i 2
		    }
		}
	    }
	    -squared {
		set current(scale) squared
		ChangeColorScale
	    }
	    -sqrt {
		set current(scale) sqrt
		ChangeColorScale
	    }
	    -source {
		incr i
		uplevel #0 source [lindex $argv $i]
	    }
	    -tile {
		set ds9(displaymode) tile
		DisplayMode
	    }
	    -tilemode {
		incr i
		set tile(mode) [lindex $argv $i]
		DisplayMode
	    }
	    -title {
		incr i
		set ds9(title) [lindex $argv $i]
		set t "SAOImage $ds9(title)"
		wm title $ds9(top) "$t"
		wm iconname $ds9(top) "$t"
	    }
	    -unix {
		incr i
		set iis(unix) [lindex $argv $i]
	    }
	    -unix_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(port) 0
	    }
	    -version {puts "$ds9(version)";QuitDS9}
	    -vertgraph {
		set graphVert(view) 1
		UpdateView
	    }
	    -novertgraph {
		set graphVert(view) 0
		UpdateView
	    }
	    -wcs {
		incr i
		set wcs(system) [lindex $argv $i]
	    }
	    -wcsformat {
		incr i
		set wcs(format) [lindex $argv $i]
	    }
	    -z1 {
		incr i
		set user(low) [lindex $argv $i]
		ScaleLimitLow [lindex $argv $i]
	    }
	    -z2 {
		incr i
		set user(high) [lindex $argv $i]
		ScaleLimitHigh [lindex $argv $i]
	    }
	    -zscale {
		set scale(mode) zscale
		ScaleMode
	    }
	    -zoom {
		incr i
		set current(zoom) [lindex $argv $i]
		ChangeZoom
	    }
	    default {
		if {[string length $item] == 1 || \
			[string range $item 0 0] != "-"} {

		    LoadCmdLine fileCount

		    LoadFits $item ""

		    if {[regexp {http://.*} $item] || \
			    [regexp {ftp://.*} $item]} {
			set ds9(lastURL) $item
		    } else {
			set ds9(lastFitsName) $item
		    }
		} else {
		    puts "Unknow command line option: $item"
		    DumpHelp
		    exit
		}
	    }
	}
	incr i
    }
}

proc LoadCmdLine {fc} {
    global ds9
    upvar $fc fileCount

    # create a new frame
    if {$fileCount != 0} {
	CreateFrame  
    }

    # go into tile mode if more than one
    if {$fileCount >= 1} {
	set ds9(displaymode) tile
	DisplayMode
    }
    incr fileCount
}

proc DumpHelp {} {
    global ds9
    global bin

    puts "DS9 version $ds9(version)"
    puts "Usage: ds9 \[options\] \[filename(s)...\] \n\
          \-?\n\
          \-align\n\
          \-array <filename>\n\
          \-blink\n\
          \-bin \t$bin(help)\n\
          \-cmap <name>\n\
          \-\[no\]colorbar\n\
          \-\[no\]contour\n\
          \-coords \[wcs|detector|amplifier|physical|image\] \[yes|no\]\n\
          \-dss <object>\n\
          \-exit\n\
          \-fifo <name>\n\
          \-fifo_only\n\
          \-geometry <value>\n\
          \-\[no\]grid\n\
          \-\[no\]horzgraph\n\
          \-\iconify\n\
          \-\[no\]info\n\
          \-\inet_only\n\
          \-invert\n\
          \-linear\n\
          \-log\n\
          \-lower\n\
          \-\[no\]magnifier\n\
          \-minmax\n\
          \-mode \[pointer|crosshair|colorbar|pan|zoom|rotate|examine\]\n\
          \-mosaicimage <filename>\n\
          \-mosaicimages <filename>\n\
          \-mosaic <filename>\n\
          \-orient \[none|x|y|xy\]\n\
          \-\[no\]panner\n\
          \-print\n\
          \-private\n\
          \-port <number>\n\
          \-port_only\n\
          \-quit\n\
          \-raise\n\
          \-rass <object>\n\
          \-regionfile <filename>\n\
          \-rotate <value>\n\
          \-scale \[linear|log|squared|sqrt\]\n\
          \-scalelims <value> <value>\n\
          \-scalemode \[minmax|zscale|user\]\n\
          \-shm <key> <size> \[<filename>\]\n\
          \-shm \[key|shmid\] <id> <size> \[<filename>\]\n\
          \-shm mosaicimage \[key|shmid\] id> <size> \[<filename>\]\n\
          \-shm mosaic \[key|shmid\] <id> <size> \[<filename>\]\n\
          \-shm array \[key|shmid\] <id> <size> <filename>\n\
          \-squared\n\
          \-sqrt\n\
          \-source <filename>\n\
          \-tile\n\
          \-tilemode \[grid|column|row\]\n\
          \-title <name>\n\
          \-unix <name>\n\
          \-unix_only\n\
          \-version\n\
          \-\[no\]vertgraph\n\
          \-visual \[pseudocolor8|truecolor8|truecolor16|truecolor24\]\n\
          \-wcs \[fk4|fk5|galactic|ecliptic\]\n\
          \-wcsformat \[degrees|sexagesimal\]\n\
          \-z1 <value>\n\
          \-z2 <value>\n\
          \-zscale\n\
          \-ztrans \[linear|log|squared|sqrt\]\n\
          \-zoom <value>\n\
"
}
