#define EVREGIONS_C "\n\
#ifndef SELF_CONTAINED\n\
#include \"regions.h\"\n\
#endif\n\
\n\
/* adding this to FilterOpen will cause this module to be loaded\n\
   for dynamic linking */\n\
static int evregno=0;\n\
void initevregions(void)\n\
{\n\
  evregno++;\n\
  return;\n\
}\n\
\n\
static int polypt(double x, double y, double* poly, int count, double ystart)\n\
{\n\
  /*\n\
    x-- x value of point being tested\n\
    y-- y value of point being tested\n\
    poly-- bounding polygon, array of length (count*2)\n\
    count-- number of edges of bounding polygon\n\
\n\
    This algorithm is from \"An Introduction to Ray Tracing\", Academic Press,\n\
    1989, edited by Andrew Glassner, pg 53\n\
    -- a point lies in a polygon if a line is extended from the point to infinite\n\
    in any direction and the number of intersections with the polygon is odd.\n\
    This is valid for both concave and convex polygons.\n\
    Points on a vertex are considered inside.\n\
    Points on a edge are considered inside.\n\
  */\n\
\n\
  /* number of crossings */\n\
  int crossings = 0;\n\
\n\
  /* init sign for first vertice */\n\
  int sign = ((poly[1] - y)>=0) ? 1 : -1;\n\
\n\
  /* for all edges */\n\
  int i;\n\
  for(i=0; i<count; i++) {\n\
    int j = (i!=(count-1)) ? i+1 : 0;\n\
\n\
    /* Vertex 1 */\n\
    double x1 = poly[i*2] - x;\n\
    double y1 = poly[i*2+1] - y;\n\
\n\
    /* Vertex 2 */\n\
    double x2 = poly[j*2] - x;\n\
    double y2 = poly[j*2+1] - y;\n\
\n\
    /* sign holder for p2 */\n\
    int nextSign = (y2>=0) ? 1 : -1;\n\
\n\
    /* This mess is an attempt to control whether the edges of a polygon\n\
       are \"in\" or \"out\". We do not want all borders to be \"in\" because that\n\
       means adjacent polygons will both have border points \"in\", violating\n\
       the rule that no photon gets counted more than once.\n\
       In general, if we are on an \"odd\" line, we are \"in\", otherwise \"out\".\n\
       However, this means horiz lines are usually \"out\", i.e., both horiz\n\
       lines of a box with no rotation would be \"out\". We fudge this by\n\
       making the ystart horiz line always \"in\". Note that this is not a\n\
       problem with vertical lines, the first will be \"in\", second is \"out\".\n\
    */\n\
    /* horiz line */\n\
    if( (y1==0) && (y2==0) ){\n\
      if( ((x2>=0) && (x1<=0)) || ((x1>=0) && (x2<=0)) ){\n\
	/* we always return true for ystart horiz lines */\n\
	if( y == ystart )\n\
	  return 1;\n\
	else\n\
	  return fmod(crossings+1,2) ? 1 : 0;\n\
      }\n\
    }\n\
    /* vertical line */\n\
    if( (x1==0) && (x2==0) ){\n\
      if( ((y2>=0) && (y1<=0)) || ((y1>=0) && (y2<=0)) )\n\
	return fmod(crossings+1,2) ? 1 : 0;\n\
    }\n\
    /* point on line */\n\
    if (y1*(x2-x1) == x1*(y2-y1)){\n\
      if( x1 && y1 )\n\
	return fmod(crossings+1,2) ? 1 : 0;\n\
    }\n\
\n\
    if (sign != nextSign) {\n\
      if (x1>0 && x2>0)\n\
	crossings++;\n\
      else if (x1>0 || x2>0) {\n\
	if (x1-(y1*(x2-x1)/(y2-y1)) > 0)\n\
	  crossings++;\n\
      }\n\
      sign = nextSign;\n\
    }\n\
  }\n\
\n\
  return fmod(crossings,2) ? 1 : 0;          /* if odd, point is inside */\n\
}\n\
\n\
static void quadeq(double a, double b, double c, \n\
	    double *x1, double *x2, int *nr, int *nc)\n\
{\n\
  double dis, q;\n\
  if(a == 0.0){\n\
    *nc = 0;\n\
    if(b == 0.0){\n\
      *nr = 0; *x1 = 0.0;\n\
    }\n\
    else{\n\
      *nr = 1; *x1 = -c / b;\n\
    }\n\
    *x2 = *x1;\n\
  }\n\
  else{\n\
    dis = b*b - 4.0 * a * c;\n\
    if( dis > 0.0 ){\n\
      *nr = 2; *nc = 0;\n\
      dis = sqrt(dis);\n\
      if( b < 0.0 ) dis = -dis;\n\
      q = -0.5 * (b + dis);\n\
      *x1 = q/a; *x2 = c/q;\n\
      if(*x1 > *x2){\n\
	q = *x1; *x1 = *x2; *x2 = q;\n\
      }\n\
    } \n\
    else if( dis == 0.0 ){\n\
      *nr = 1; *nc = 0; *x1 = - 0.5 * b / a; *x2 = *x1;\n\
    }\n\
    else{\n\
      *nr = 0; *nc = 2; *x1 = - 0.5 * b / a; *x2 = 0.5 * sqrt(-dis) / a;\n\
    }\n\
  }\n\
}\n\
\n\
static int corner_vertex(int index, int width, int height,\n\
			 double *x, double *y)\n\
{\n\
  switch (index) {\n\
  case 1:\n\
    *x = 0.0;\n\
    *y = height + 1;\n\
    break;\n\
  case 2:\n\
    *x = 0.0;\n\
    *y = 0.0;\n\
    break;\n\
  case 3:\n\
    *x = width + 1;\n\
    *y = 0.0;\n\
    break;\n\
  case 4:\n\
    *x = width + 1;\n\
    *y = height + 1;\n\
  default:\n\
    break;\n\
  }\n\
  index = index + 1;\n\
  if(index > 4) index = 1;\n\
  return(index);\n\
}\n\
\n\
static int pie_intercept(int width, double height, double xcen, double ycen,\n\
			 double angle, double *xcept, double *ycept)\n\
{\n\
  double angl, slope;	/* l: angle and slope of ray */\n\
  angl = angle;\n\
  /* put angles in normal range */\n\
  while (angl < 0.0)\n\
    angl = angl + 360.0;\n\
  while (angl >= 360.0)\n\
    angl = angl - 360.0;\n\
  /* check for a horizontal angle */\n\
  if(fabs(angl - 90.0) < SMALL_NUMBER) {\n\
    *xcept = 0.0;\n\
    *ycept = ycen;\n\
    return(2);\n\
  }\n\
  if(fabs(angl - 270.0) < SMALL_NUMBER) {\n\
    *xcept = width + 1;\n\
    *ycept = ycen;\n\
    return(4);\n\
  }\n\
  /* convert to a Cartesian angle */\n\
  angl = angl + 90.0;\n\
  if(angl >= 360.0)\n\
    angl = angl - 360.0;\n\
  if(angl < 180.0) {\n\
    *ycept = height + 1;\n\
    /* rule out vertical line */\n\
    if(fabs(angl - 90.0) < SMALL_NUMBER) {\n\
      *xcept = xcen;\n\
      return(1);\n\
    }\n\
  } else {\n\
    *ycept = 0.0;\n\
    /* rule out vertical line */\n\
    if(fabs(angl - 270.0) < SMALL_NUMBER) {\n\
      *xcept = xcen;\n\
      return(3);\n\
    }\n\
  }\n\
  /* convert to radians */\n\
  angl = (angl / 180.0) * PI;\n\
  /* calculate slope */\n\
  slope = tan(angl);\n\
  /* calculate intercept with designated y edge */\n\
  *xcept = xcen + ((*ycept - ycen) / slope);\n\
  if(*xcept < 0) {\n\
    *ycept = (ycen - (xcen * slope));\n\
    *xcept = 0.0;\n\
    return(2);\n\
  } else if(*xcept > (width + 1)) {\n\
    *ycept = (ycen + ((width + 1 - xcen) * slope));\n\
    *xcept = width + 1;\n\
    return(4);\n\
  } else {\n\
    if(*ycept < height)\n\
      return(3);\n\
    else\n\
      return(1);\n\
  }\n\
}\n\
\n\
/* ***************************** shapes ********************************** */\n\
\n\
int evannulus(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	      double xcen, double ycen, double ri, double ro)\n\
{\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    g->shapes[sno].ystart = ycen - ro;\n\
    g->shapes[sno].ystop = ycen + ro;\n\
    g->shapes[sno].r1sq = ri * ri;\n\
    g->shapes[sno].r2sq = ro * ro;\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		    &&\n\
       (((xcen-x)*(xcen-x))+((ycen-y)*(ycen-y))<=g->shapes[sno].r2sq)	    &&\n\
       (((xcen-x)*(xcen-x))+((ycen-y)*(ycen-y))>g->shapes[sno].r1sq)) == flag ){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
int evbox(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	  double xcen, double ycen, double xwidth, double yheight,\n\
	  double angle)\n\
{\n\
  int i;\n\
  double angl;			 /* l: Cartesian angle in radians */\n\
  double half_width, half_height;/* l: radii (1/2 width and height) */\n\
  double cosangl, sinangl;	 /* l: sine, cosine of the Cartesian angle */\n\
  double hw_cos, hw_sin;	 /* l: products of half_width with sin, cos */\n\
  double hh_cos, hh_sin;	 /* l: products of half_height with sin, cos */\n\
\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    /* convert to a Cartesian angle; save angle for use in multi or slices */\n\
    angl = angle + 90.0;\n\
    while (angl >= 360.0) angl = angl - 360.0;\n\
    /* convert to radians */\n\
    angl = (angl / 180.0) * PI;\n\
    sinangl = sin (angl);\n\
    cosangl = cos (angl);\n\
    /* since we rotate by 90.0 degrees to get from astro angle to cartesian, */\n\
    /* we also need to switch the width and height. we do this secretly so */\n\
    /* that the display will turn out right, by doing it in the half terms */\n\
    half_width = yheight / 2.0;\n\
    half_height = xwidth / 2.0;\n\
    hw_cos = half_width * cosangl;\n\
    hw_sin = half_width * sinangl;\n\
    hh_cos = half_height * cosangl;\n\
    hh_sin = half_height * sinangl;\n\
    g->shapes[sno].pts = (double *)malloc(8*sizeof(double));\n\
    g->shapes[sno].pts[0] = xcen - hw_cos - hh_sin;\n\
    g->shapes[sno].pts[1] = ycen - hw_sin + hh_cos;\n\
    g->shapes[sno].pts[2] = xcen + hw_cos - hh_sin;\n\
    g->shapes[sno].pts[3] = ycen + hw_sin + hh_cos;\n\
    g->shapes[sno].pts[4] = xcen + hw_cos + hh_sin;\n\
    g->shapes[sno].pts[5] = ycen + hw_sin - hh_cos;\n\
    g->shapes[sno].pts[6] = xcen - hw_cos + hh_sin;\n\
    g->shapes[sno].pts[7] = ycen - hw_sin - hh_cos;\n\
    g->shapes[sno].npt = 8;\n\
    /* now find the y limits */\n\
    if( g->shapes[sno].npt ){\n\
      g->shapes[sno].ystart = g->shapes[sno].pts[1];\n\
      g->shapes[sno].ystop = g->shapes[sno].ystart;\n\
      for(i=1; i<g->shapes[sno].npt; i+=2){\n\
	if(g->shapes[sno].pts[i] > g->shapes[sno].ystop)\n\
	  g->shapes[sno].ystop = g->shapes[sno].pts[i];\n\
	if(g->shapes[sno].pts[i] < g->shapes[sno].ystart)\n\
	  g->shapes[sno].ystart = g->shapes[sno].pts[i];\n\
      }\n\
    }\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		&&\n\
       polypt(x, y, g->shapes[sno].pts, g->shapes[sno].npt/2, \n\
	      g->shapes[sno].ystart)) == flag				){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
int evcircle(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	     double xcen, double ycen, double radius)\n\
{\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    g->shapes[sno].ystart = ycen - radius;\n\
    g->shapes[sno].ystop = ycen + radius;\n\
    g->shapes[sno].r1sq = radius * radius;\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		    &&\n\
      (((xcen-x)*(xcen-x))+((ycen-y)*(ycen-y))<=g->shapes[sno].r1sq)) == flag  ){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
int evellipse(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	      double xcen, double ycen, double xrad, double yrad, double angle)\n\
{\n\
  double yhi, yoff;\n\
  double b, c;\n\
  double b_partial, c_partial;\n\
  double xboff, xfoff;\n\
  int nr, nc;\n\
\n\
  /* use circle if possible */\n\
  if( xrad == yrad )\n\
    return(evcircle(g, rno, sno, flag, x, y, xcen, ycen, xrad));\n\
\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    /* set worst case limits (xrad axis parallel to vertical axis) */\n\
    /* convert to a Cartesian angle; save \"angle\" for use by other routines */\n\
    g->shapes[sno].angl = angle + 90.0;\n\
    while( g->shapes[sno].angl >= 360.0 )\n\
      g->shapes[sno].angl = g->shapes[sno].angl - 360.0;\n\
    /* convert to radians */\n\
    g->shapes[sno].angl = (g->shapes[sno].angl / 180.0) * PI;\n\
    g->shapes[sno].sinangl = sin(g->shapes[sno].angl);\n\
    g->shapes[sno].cosangl = cos(g->shapes[sno].angl);\n\
    /* calculate approximate y limits */\n\
    /* choose lesser of containing rotbox and circle */\n\
    yhi = fabs(g->shapes[sno].sinangl * yrad) + fabs(g->shapes[sno].cosangl * xrad);\n\
    yhi = min(yhi, max(yrad, xrad));\n\
    g->shapes[sno].ystart = ycen - yhi;\n\
    g->shapes[sno].ystop = ycen + yhi;\n\
    /* prepare partials for quadratic equation solutions to coordinates */\n\
    g->shapes[sno].cossq = g->shapes[sno].cosangl * g->shapes[sno].cosangl;\n\
    g->shapes[sno].sinsq = g->shapes[sno].sinangl * g->shapes[sno].sinangl;\n\
    /* because we rotate by 90.0 degrees to get from astro angle to */\n\
    /* cartesian, we also need to switch the x and y axes. we do this */\n\
    /* secretly so that the display will turn out right, by doing it in */\n\
    /* the sq terms */\n\
    g->shapes[sno].xradsq = yrad * yrad;\n\
    g->shapes[sno].yradsq = xrad * xrad;\n\
    /* fill in as much of a,b,c as we can */\n\
    g->shapes[sno].a = (g->shapes[sno].cossq / g->shapes[sno].xradsq) + \n\
      		    (g->shapes[sno].sinsq / g->shapes[sno].yradsq);\n\
  }\n\
  if( ((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop)) ){\n\
    b_partial = (2.0 * g->shapes[sno].sinangl) * \n\
      ((g->shapes[sno].cosangl / g->shapes[sno].xradsq) - \n\
       (g->shapes[sno].cosangl / g->shapes[sno].yradsq));\n\
    c_partial = (g->shapes[sno].sinsq / g->shapes[sno].xradsq) + \n\
      (g->shapes[sno].cossq / g->shapes[sno].yradsq);\n\
    yoff = y - ycen;\n\
    b = b_partial * yoff;\n\
    c = (c_partial * yoff * yoff) - 1.0;\n\
    /* solve quadratic */\n\
    quadeq (g->shapes[sno].a, b, c, &xboff, &xfoff, &nr, &nc);\n\
    /* if real roots */\n\
    if( nr != 0 ) {\n\
      if( ((x>=(xcen+xboff)) && (x<=(xcen+xfoff))) == flag ){\n\
	if( flag ) g->rid = rno;\n\
	return 1;\n\
      }\n\
      else\n\
	return 0;\n\
    }\n\
    else\n\
      return !flag;\n\
  }\n\
  return !flag;\n\
}\n\
\n\
int evfield(GFilt g, int rno, int sno, int flag, double x, double y)\n\
{\n\
  if( flag ){\n\
    g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
\n\
int evline(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	   double x1, double y1, double x2, double y2)\n\
{\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    g->shapes[sno].ystart = min(y1,y2);\n\
    g->shapes[sno].ystop = max(y1,y2);\n\
    g->shapes[sno].x1 = x1;\n\
    g->shapes[sno].x2 = x2;\n\
    g->shapes[sno].y1 = y1;\n\
    if( feq(y1,y2) ){\n\
      g->shapes[sno].xonly = 1;\n\
      g->shapes[sno].invslope = 0;\n\
    }\n\
    else{\n\
      g->shapes[sno].xonly = 0;\n\
      g->shapes[sno].invslope = (x1 - x2) / (y1 - y2);\n\
    }\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		 &&\n\
      ((!g->shapes[sno].xonly &&\n\
       feq((((y-g->shapes[sno].y1)*g->shapes[sno].invslope)+g->shapes[sno].x1),x)) ||\n\
      (g->shapes[sno].xonly &&\n\
       ((x>=g->shapes[sno].x1)&&(x<=g->shapes[sno].x2))))) == flag){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
int evpie(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	  double xcen, double ycen, double angle1, double angle2)\n\
{\n\
  int i;\n\
  int width, height;		/* l: image mask width and height */\n\
  double sweep;			/* l: sweep between cut angles */\n\
  int intrcpt1, intrcpt2;	/* l: side intercepted by each cut */\n\
  double x2, y2;		/* l: coordinates of second intercept */\n\
\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    /* we have to fake width and height, since we do not know them! */\n\
    width = LARGE_NUMBER;\n\
    height = LARGE_NUMBER;\n\
    /* start listing vertices of polygon */\n\
    g->shapes[sno].pts = (double *)malloc(14*sizeof(double));\n\
    g->shapes[sno].pts[0] = xcen;\n\
    g->shapes[sno].pts[1] = ycen;\n\
    sweep = angle2 - angle1;\n\
    /* if sweep is too small to be noticed, don't bother */\n\
    if(fabs(sweep) < SMALL_NUMBER)\n\
      return !flag;\n\
    if (sweep < 0.0) sweep = sweep + 360.0;\n\
    intrcpt1 = pie_intercept(width, height, xcen, ycen, angle1,\n\
			     &(g->shapes[sno].pts[2]), &(g->shapes[sno].pts[3]));\n\
    intrcpt2 = pie_intercept(width, height, xcen, ycen, angle2,\n\
			     &x2, &y2);\n\
    g->shapes[sno].npt = 4;\n\
    /* if angles intercept same side and slice is between them, no corners */\n\
    /* else, mark corners until reaching side with second angle intercept */\n\
    if((intrcpt1 != intrcpt2) || (sweep > 180.0)){\n\
      do{\n\
	intrcpt1 = corner_vertex(intrcpt1, width, height,  \n\
				 &(g->shapes[sno].pts[g->shapes[sno].npt]),\n\
				 &(g->shapes[sno].pts[g->shapes[sno].npt+1]));\n\
	g->shapes[sno].npt = g->shapes[sno].npt + 2;\n\
      }while(intrcpt1 != intrcpt2);\n\
    }\n\
    g->shapes[sno].pts[g->shapes[sno].npt] = x2;\n\
    g->shapes[sno].pts[g->shapes[sno].npt+1] = y2;\n\
    g->shapes[sno].npt = g->shapes[sno].npt + 2;\n\
    /* now find the y limits */\n\
    if( g->shapes[sno].npt ){\n\
      g->shapes[sno].ystart = g->shapes[sno].pts[1];\n\
      g->shapes[sno].ystop = g->shapes[sno].ystart;\n\
      for(i=1; i<g->shapes[sno].npt; i+=2){\n\
	if(g->shapes[sno].pts[i] > g->shapes[sno].ystop)\n\
	  g->shapes[sno].ystop = g->shapes[sno].pts[i];\n\
	if(g->shapes[sno].pts[i] < g->shapes[sno].ystart)\n\
	  g->shapes[sno].ystart = g->shapes[sno].pts[i];\n\
      }\n\
    }\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		&&\n\
       polypt(x, y, g->shapes[sno].pts, g->shapes[sno].npt/2, \n\
	      g->shapes[sno].ystart)) == flag				){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
int evqtpie(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	    double xcen, double ycen, double angle1, double angle2)\n\
{\n\
  return evpie(g, rno, sno, flag, x, y, xcen, ycen, angle1, angle2);\n\
}\n\
\n\
int evpoint(GFilt g, int rno, int sno, int flag, double x, double y,\n\
	    double xcen, double ycen)\n\
{\n\
  if( ((x==xcen) && (y==ycen)) == flag ){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
\n\
#ifdef __STDC__\n\
int\n\
evpolygon(GFilt g, int rno, int sno, int flag, double x, double y, ...)\n\
{\n\
  int i, maxpts;\n\
  va_list args;\n\
  va_start(args, y);\n\
#else\n\
int evpolygon(va_alist) va_dcl\n\
{\n\
  GFilt g;\n\
  int rno, sno, flag;\n\
  double x, y;\n\
  int i, maxpts;\n\
  va_list args;\n\
  va_start(args);\n\
  g  = va_arg(args, GFilt);\n\
  rno  = va_arg(args, int);\n\
  sno  = va_arg(args, int);\n\
  flag  = va_arg(args, int);\n\
  x  = va_arg(args, double);\n\
  y  = va_arg(args, double);\n\
#endif\n\
  if( !g->shapes[sno].init ){\n\
    g->shapes[sno].init = 1;\n\
    /* allocate space for x,y arguments */\n\
    maxpts = MASKINC;\n\
    g->shapes[sno].pts = (double *)malloc(maxpts*sizeof(double));\n\
    /* gather up arguments */\n\
    g->shapes[sno].npt = 0;\n\
    while( 1 ){\n\
      if( g->shapes[sno].npt >= maxpts ){\n\
	maxpts += MASKINC;\n\
	g->shapes[sno].pts = (double *)realloc(g->shapes[sno].pts,\n\
					    maxpts*sizeof(double));\n\
      }\n\
      g->shapes[sno].pts[g->shapes[sno].npt] = va_arg(args, double);\n\
      /* two negatives in a row means we are at end of args */\n\
      if( ((g->shapes[sno].npt % 2) == 1)			&&\n\
	  feq(g->shapes[sno].pts[g->shapes[sno].npt],PSTOP)	&&\n\
	  feq(g->shapes[sno].pts[g->shapes[sno].npt-1],PSTOP) ){\n\
	g->shapes[sno].npt--;\n\
	break;\n\
      }\n\
      g->shapes[sno].npt++;\n\
    }\n\
    va_end(ap);\n\
    /* realloc to actual size */\n\
    g->shapes[sno].pts = (double *)realloc(g->shapes[sno].pts,\n\
					g->shapes[sno].npt*sizeof(double));\n\
    /* now find the y limits */\n\
    if( g->shapes[sno].npt ){\n\
      g->shapes[sno].ystart = g->shapes[sno].pts[1];\n\
      g->shapes[sno].ystop = g->shapes[sno].ystart;\n\
      for(i=1; i<g->shapes[sno].npt; i+=2){\n\
	if(g->shapes[sno].pts[i] > g->shapes[sno].ystop)\n\
	  g->shapes[sno].ystop = g->shapes[sno].pts[i];\n\
	if(g->shapes[sno].pts[i] < g->shapes[sno].ystart)\n\
	  g->shapes[sno].ystart = g->shapes[sno].pts[i];\n\
      }\n\
    }\n\
  }\n\
  if( (((y>=g->shapes[sno].ystart) && (y<=g->shapes[sno].ystop))		&&\n\
       polypt(x, y, g->shapes[sno].pts, g->shapes[sno].npt/2, \n\
	      g->shapes[sno].ystart)) == flag				){\n\
    if( flag ) g->rid = rno;\n\
    return 1;\n\
  }\n\
  else\n\
    return 0;\n\
}\n\
"
