#ifndef __fitsfile_h__
#define __fitsfile_h__

#include <string.h>
#include <tcl.h>

#include "head.h"

class FitsFile {
public:
  enum ScanMode {RELAX, EXACT};
  enum ArchType {NATIVE,BIGENDIAN,LITTLEENDIAN};

protected:
  FitsHead* head_;           // pointer to header
  FitsHead* primary_;        // pointer to primary header
  int managePrimary_;        // flag, true if we manage primary header

  void* data_;               // pointer to raw data
  int byteswap_;             // flag, true if byteswap is needed
  int valid_;                // flag, true if file is valid

  char* pName_;              // parsed file name
  char* pExt_;               // parsed ext name
  int pIndex_;               // parsed ext number
  char* pFilter_;            // unparsed filter spec
  char* pBinX_;              // parsed bin table x col name
  char* pBinY_;              // parsed bin table y col name
  char* pBinZ_;              // parsed bin table z col name

  int pWidth_;               // parsed array width
  int pHeight_;              // parsed array height
  int pBitpix_;              // parsed array bitpix
  int pSkip_;                // parsed array skip size
  ArchType pArch_;           // parsed array arch type

protected:
  void parse(const char*);
  int findEnd(const char*);
  void setArrayByteSwap();
  int validArrayParams();

public:
  FitsFile();
  virtual ~FitsFile();

  void error(const char*);
  void* data() {return data_;}
  FitsHead* head() {return head_;}
  FitsHead* primary() {return primary_;}
  int isValid() {return valid_;}
  int isImage() {return head_ ? head_->isImage() : 0;}
  int isTable() {return head_ ? head_->isTable() : 0;}
  int byteswap() {return byteswap_;}

  void setpName(const char*);
  void setpExt(const char*);
  void setpIndex(int i) {pIndex_ = i;}
  void setpFilter(const char*);
  void setpBinX(const char*);
  void setpBinY(const char*);
  void setpBinZ(const char*);
  void setpBinXY(const char* x, const char* y) {setpBinX(x); setpBinY(y);}
  void setpBinXYZ(const char* x, const char* y, const char* z)
    {setpBinX(x); setpBinY(y); setpBinZ(z);}

  const char* pName() {return pName_;}
  const char* pExt() {return pExt_;}
  int pIndex() {return pIndex_;}
  const char* pFilter() {return pFilter_;}
  const char* pBinX() {return pBinX_;}
  const char* pBinY() {return pBinY_;}
  const char* pBinZ() {return pBinZ_;}

  void setpXdim(int i) {pWidth_ = i;}
  void setpYdim(int i) {pHeight_ = i;}
  void setpBitpix(int b) {pBitpix_ = b;}
  void setpSkip(int s) {pSkip_ = s;}
  void setpArch(ArchType a) {pArch_ = a;}

  int find(const char* name);
  int getLogical(const char* name, int def);
  int getInteger(const char* name, int def);
  double getReal(const char* name, double def);
  void getComplex(const char* name, double* real, double* img, 
		  double rdef, double idef);
  char* getString(const char* name);
  char* getComment(const char* name);

  int saveFitsFile(const char*);
  int saveFitsChannel(Tcl_Interp*, const char*);
};

#endif







