#include <sys/shm.h>

#include "share.h"

FitsShareID::FitsShareID(int shmid, int sz, const char* filter)
{
  parse(filter);

  // Attach the memory segment

  mapsize_ = sz;

  if ((int)(mapdata_ = (char*)shmat(shmid, 0, SHM_RDONLY)) == -1) {
    cerr << "Fitsy++ Internal Error: shmat failed" << endl;
    return;
  }

  // so far so good
  valid_ = 1;
}

FitsShareID::~FitsShareID()
{
  shmdt(mapdata_);
}

FitsShareKey::FitsShareKey(int key, int sz, const char* filter)
{
  parse(filter);

  // Attach the memory segment

  mapsize_ = sz;
  int shmid;
  if ((shmid = shmget(key, sz, 0)) < 0)
    return;

  if ((int)(mapdata_ = (char*)shmat(shmid, 0, SHM_RDONLY)) == -1) {
    cerr << "Fitsy++ Internal Error: shmat failed" << endl;
    return;
  }

  // so far so good
  valid_ = 1;
}

FitsShareKey::~FitsShareKey()
{
  shmdt(mapdata_);
}

FitsFitsShare::FitsFitsShare(ScanMode mode) : FitsMap()
{
  if (valid_)
    if (mode == EXACT || pExt_ || pIndex_)
      processExact();
    else
      processRelax();
}

FitsArrShare::FitsArrShare() : FitsMap()
{
  if (valid_)
    processArray();
}

FitsMosaicShare::FitsMosaicShare() : FitsMap()
{
  if (valid_)
    processMosaic();
}

FitsFitsShareID::FitsFitsShareID(int shmid, int sz,
				   const char* filter, ScanMode mode) 
  : FitsShareID(shmid, sz, filter), FitsFitsShare(mode) {}

FitsFitsShareKey::FitsFitsShareKey(int key, int sz,
				   const char* filter, ScanMode mode) 
  : FitsShareKey(key, sz, filter), FitsFitsShare(mode) {}

FitsArrShareID::FitsArrShareID(int shmid, int sz, const char* filter) 
  : FitsShareID(shmid, sz, filter), FitsArrShare() {}

FitsArrShareKey::FitsArrShareKey(int key, int sz,
				 const char* filter) 
  : FitsShareKey(key, sz, filter), FitsArrShare() {}

FitsMosaicShareID::FitsMosaicShareID(int shmid, int sz)
  : FitsShareID(shmid, sz, ""), FitsMosaicShare() {}

FitsMosaicShareKey::FitsMosaicShareKey(int key, int sz)
  : FitsShareKey(key, sz, ""), FitsMosaicShare() {}
