#include "basebox.h"
#include "frame.h"
#include "util.h"

BaseBox::BaseBox(Frame* p, const Vector& ctr, double a,
		 const char* clr, const char* f, const char* t, 
		 unsigned short prop)
  : BaseMarker(p, ctr, 4, clr, f, t, prop)
{
  size = NULL;
  angle = a;

  if (!parent->isIIS())
    flip = FlipY();
}

BaseBox::~BaseBox()
{
  if (size)
    delete [] size;
}

void BaseBox::updateCoords(Matrix& m)
{
  for (int i=0; i<annuli; i++)
    size[i] *= Scale(m);
  Marker::updateCoords(m);
}

void BaseBox::rotate(const Vector& v, int h)
{
  // v is in ref coords
  // handles are in canvas coords

  Vector a = v - center;
  Vector b = (handle[h-1] * parent->canvasToRef) - center;

  if (parent->isIIS())
    angle -= a.angle() - b.angle();
  else
    angle += a.angle() - b.angle();

  updateBBox();
  doCallBacks(&rotateCB);
}

void BaseBox::setAngle(double a)
{
  angle = a;
  updateBBox();

  doCallBacks(&rotateCB);
}

int BaseBox::isIn(const Vector& v)
{
  // v is in canvas coords

  Vector s = size[annuli-1];
  if (properties & FIXED)
    s /= parent->getZoom();

  Vector p = v * 
    parent->canvasToRef * Translate(-center) * flip * Rotate(-angle);

  if (p[0]<-s[0]/2 || p[0]>s[0]/2 || p[1]<-s[1]/2 || p[1]>s[1]/2)
    return 0;
  else
    return 1;
}

// private

void BaseBox::calcPoints(double zoom)
{
  Matrix m;
  if (properties & FIXED)
    m = Scale(1/zoom) * Rotate(angle) * flip * Translate(center);
  else
    m = Rotate(angle) * flip * Translate(center);

  for (int i=0; i<annuli; i++) {
    vertices[i][0] = Vector(-size[i][0]/2.,-size[i][1]/2.) * m;
    vertices[i][1] = Vector( size[i][0]/2.,-size[i][1]/2.) * m;
    vertices[i][2] = Vector( size[i][0]/2., size[i][1]/2.) * m;
    vertices[i][3] = Vector(-size[i][0]/2., size[i][1]/2.) * m;
  }
}

