#ifndef __frame_h__
#define __frame_h__

#include <fstream.h>

#include <tk.h>

#include "widget.h"
#include "vector.h"
#include "colorscale.h"
#include "marker.h"
#include "hist.h"
#include "list.h"

//#define SCALESIZE 65536
//#define SCALESIZE 32768
#define SCALESIZE 16384

#define DebugPerf 0
#define DebugWCS 0
#define DebugMosaic 0

class FitsImage;
class ContourPlot;
class Grid;
class Attribute;
class Marker;
class BaseBox;
class BaseCircle;
class BaseEllipse;
class BaseMarker;
class Circle;
class Annulus;
class Ellipse;
class EllipseAnnulus;
class Box;
class BoxAnnulus;
class Polygon;
class Point;
class Line;
class Text;
class Ruler;
class CirclePoint;
class BoxPoint;
class DiamondPoint;
class CrossPoint;
class ExPoint;
class ArrowPoint;
class BoxCirclePoint;

// Frame

class Frame : public Widget {
  friend FitsImage;
  friend ContourPlot;
  friend Grid;
  friend Attribute;
  friend Marker;
  friend BaseBox;
  friend BaseCircle;
  friend BaseEllipse;
  friend BaseMarker;
  friend Circle;
  friend Annulus;
  friend Ellipse;
  friend EllipseAnnulus;
  friend Box;
  friend BoxAnnulus;
  friend Polygon;
  friend Point;
  friend Line;
  friend Text;
  friend Ruler;
  friend CirclePoint;
  friend BoxPoint;
  friend DiamondPoint;
  friend CrossPoint;
  friend ExPoint;
  friend ArrowPoint;
  friend BoxCirclePoint;

public:
  enum Orientation {NORMAL=0, X=1, Y=2, XY=3};
  enum Precision {DEFAULT, FIXED, SCIENTIFIC, INTEGER};
  enum ColorScaleType {LINEARSCALE, LOGSCALE, SQUAREDSCALE, 
		       SQRTSCALE, IISSCALE};
  enum CoordSystem {WINDOW, CANVAS, WIDGET, USER, REF, DATA, PANNER,
		    IMAGE, PHYSICAL, AMPLIFIER, DETECTOR, 
		    EQUATORIAL_FK4, EQUATORIAL_FK5, ECLIPTIC, GALACTIC, 
		    LINEAR, ICRS, NATIVEWCS};
  enum CoordFormat {PIXELS, DEGREES, SEXAGESIMAL, ARCMIN, ARCSEC};
  enum ClipScope {GLOBAL, LOCAL};
  enum ClipMode {MINMAX, ZSCALE, USERCLIP};
  enum MinMaxMode {SCAN, SAMPLE, DATAMIN, IRAFMIN};
  enum MagnifierMode {FAST, ACCURATE};
  enum MemType {ALLOC, CHANNEL, MMAP, SHARE};
  enum MarkerFormat {DS9, CIAO, FTOOLS, SAOTNG, SAOIMAGE, PROS};
  enum FileNameType {BASIC, FULL};
  enum PSColorSpace {BW, GRAY, RGB, CMYK};
  enum ShmType {SHMID,KEY};
  enum Delim {NL, SEMICOLON};
  enum ColormapMode {CENTER, CLICK};
  enum ContourMethod {AVERAGE, MAX};

protected:
  FitsImage* fits;           // list of fits images
  int fitsCount;             // count of fits images
  ColorScale* colorScale;    // current color scale
  unsigned char* colorCells; // current color values
  unsigned short* indexCells; // current color indices

  XImage* baseXImage;        // ximage of image

  double rotation;           // current image rotation angle

  double rotateRotation;     // tmp cursor for interactive rotation
  XImage* rotateSrcXM;       // rotate src ximage
  XImage* rotateDestXM;      // rotate dest ximage
  Pixmap rotatePM;           // rotate pixmap
  GC rotateGCXOR;            // GC for interactive rotation

  int cmapID;                // current colormap id
  float bias;                // current colormap bias
  float contrast;            // current colormap contrast
  int invert;                // current colormap invert
  int cmapArea;              // current colormap update size
  ColormapMode cmapMode;     // current colormap update mode
  Vector cmapOffset;         // current colormap update offset

  int colorCount;            // number of dynamic colors
  ColorScaleType colorScaleType; // color scale type

  XImage* magnifierXImage;   // ximage for magnifier
  int magnifierWidth;        // magnifier width
  int magnifierHeight;       // magnifier height
  double magnifierZoom;      // magnifier zoom factor
  Vector magnifierCursor;    // we need to save the last center used

  int psInterpolate;         // postscript level 2 interpolation flag
  int psResolution;          // postscript resolution

  Vector iisLastCursor;      // iis cursor state info
  Vector iisSaveCursor;
  Window iisSaveRoot;
  int iisInteractive;
				  
  Matrix canvasToRef;
  Matrix canvasToUser;
  Matrix canvasToWidget;
  Matrix canvasToWindow;
  Matrix refToCanvas;
  Matrix refToPanner;
  Matrix refToUser;
  Matrix refToWidget;
  Matrix refToWindow;
  Matrix pannerToRef;
  Matrix pannerToUser;
  Matrix userToCanvas;
  Matrix userToRef;
  Matrix userToMagnifier;
  Matrix userToMagnifierCursor;
  Matrix userToPanner;
  Matrix userToWidget;
  Matrix widgetToCanvas;
  Matrix widgetToRef;
  Matrix widgetToUser;
  Matrix widgetToWindow;
  Matrix windowToCanvas;
  Matrix windowToRef;
  Matrix windowToWidget;
  Matrix ximageToMagnifier;

protected:
  enum UpdateType {MATRIX, BASE, PIXMAP, NOUPDATE};
  enum MosaicExtType {MOSAICMAP,MOSAICALLOC,MOSAICCHANNEL};

  char* name;                // object name
  FitsImage* currentFits;    // current fits image (for mosaic)
  Pixmap basePixmap;         // pixmap of image
				  
  UpdateType needsUpdate;           // updateFlags

  Vector cursor;             // current cursor position in widget coords
  double zoom;               // current image zoom value
  Orientation orientation;   // current image orientation
  Matrix orientationMatrix;  // current image orientation matrix

  int wcsAlignment;          // flag to indicate wcs alignment
  double wcsZoom;            // wcs image zoom factor
  double wcsRotation;        // wcs image rotation angle
  Orientation wcsOrientation; // wcs image orientation
  Matrix wcsOrientationMatrix; // wcs image orientation matrix

  Pixmap panPM;              // tmp pixmap for pan motion
  Vector panCursor;          // tmp cursor for interactive panning
  GC panGCXOR;               // GC for interactive panning

  ClipScope clipScope;       // color scale clip scope
  ClipMode clipMode;         // color scale clip mode
  MinMaxMode mmMode;         // method to use for determining minmax

  double low;                // low cut level for all data
  double high;               // high cut level for all data

  double min;                // min for all data
  double max;                // max for all data
  int scanOnLoad;            // flag

  double uLow;               // low cut via user for all data
  double uHigh;              // high cut via user for all data

  float zContrast;           // zscale slope transfer function
  int zSampleSize;           // zscale optimal sample size
  int zSampleLine;           // zscale number of lines to sample
			     // for median scaling

  int mmIncr;                // minmax sampling incr

  int histBins;              // number of bins for histogram

  Vector crosshair;          // crosshair location (in image coords)
  int useCrosshair;          // flag to use crosshair
  GC crosshairGC;            // crosshair gc
  GC crosshairGCXOR;         // crosshair gcxor

  GC highliteGC;             // gc for frame highlite

  List<Marker> markers;				  
  GC markerGC;               // marker gc
  GC markerGCXOR;            // marker gcxor
  GC markerGCXORPoint;       // marker gcxor
  GC markerGCXORRuler;       // marker gcxor
  Vector markerBegin;        // marker move begin in image coords
  Marker* editMarker;        // pointer to marker currently being edited
  Marker* rotateMarker;      // pointer to marker currently being rotated
  Vector regionBegin;        // select region begin in canvas coords
  Vector regionEnd;          // select region end in canvas coords

  Pixmap pannerPixmap;       // pixmap for panner
  XImage* pannerXImage;      // ximage for panner
  int pannerWidth;           // panner width
  int pannerHeight;          // panner height
  char pannerName[32];       // panner widget name
  int usePanner;             // panner status

  Pixmap magnifierPixmap;    // pixmap for magnifier
  char magnifierName[32];    // magnifer widget name
  MagnifierMode magnifierMode; // magnifier algorithm
  int useMagnifier;          // magnifer status
  int useMagnifierGraphics;    // magnifer marker status
  int useMagnifierCursor;    // magnifier cursor status
  GC magnifierGC;            // GC for magnifier
				  
  int useHighlite;           // flag to draw highlite box

  ContourPlot* contour;
  GC contourGC;

  Grid* grid;
  GC gridGC;

  int psLevel;               // postscript level
  PSColorSpace psColorSpace; // postscript color space

  FitsHist::Function binFunction; // bining operation
  double binFactor;        // bining factor
  int binBufferSize;       // size of bin buffer

protected:
  void update(UpdateType);         // schedule update

  virtual void updateColorScale() =0;
  virtual void fillXImage(XImage*, Matrix& (FitsImage::*)()) =0;
  virtual void fillXImageMosaic(XImage*, Matrix& (FitsImage::*)()) =0;
  virtual void clearPixmap(Pixmap, XImage*) =0;
  virtual void updateMagnifierReplicate(const Vector&) =0;
  virtual void rotateMotion() =0;

  void updateColorCells(unsigned short*, unsigned char*, int);

  void psLevel1(PSColorSpace, int, int, float);
  virtual void psLevel2(PSColorSpace, int, int, float) =0;
  Matrix psMatrix(float, int, int);

  int isIIS();

private:
  void load(FitsImage*);
  void loadMosaic(FitsImage*);
  void loadMosaicExt(MosaicExtType, const char*, FitsImage*);
  FitsImage* loadMosaicExtNext(MosaicExtType, const char*, FitsFile*);

  int postscriptProc(int);   // generate postscript

  void update(UpdateType,BBox);    // schedule update
  void updateNow(UpdateType);      // schedule update
  void updateNow(UpdateType,BBox); // schedule update

  void invalidPixmap();      // update all pixmaps
  int updatePixmap(const BBox&); // renders image/graphics into pixmap
  void updateMatrices();     // recalculate matrices
  void updateBase(const BBox&);    // generate base pixmap
  void updatePM(const BBox&);      // update part of pixmap
  void updateGCs();          // update gcs

  void ximageToPixmap(Pixmap, XImage*, int, int, Matrix& (FitsImage::*)());

  void updatePanner();       // render panner
  double calcPannerZoom();  // calculate the zoom factor for the panner
  void pannerPush();         // send panner and update message
  void updateMagnifier(); // render magnifier
  void updateMagnifier(const Vector&); // render magnifier
  void updateMagnifierMatrix();
  void updateMagnifierZoom(const Vector&); // render magnifier
  void renderMagnifierCursor(const Vector&); // render magnifier cursor

  void getClip(const Vector&, Matrix& (FitsImage::*)(), Precision);
  void getFitsFileName(const Vector&, Matrix& (FitsImage::*)(), FileNameType);
  void getFitsObjectName(const Vector&, Matrix& (FitsImage::*)());
  void getInfo(const Vector&, CoordSystem,
	       Matrix& (FitsImage::*)(), Matrix& (FitsImage::*)(),
	       CoordSystem, CoordFormat, char*);
  void getInfoSingle(const Vector&, CoordSystem,
		     Matrix& (FitsImage::*)(), Matrix& (FitsImage::*)(),
		     CoordSystem, CoordFormat, char*);
  void getInfoMosaic(const Vector&, CoordSystem,
		     Matrix& (FitsImage::*)(), Matrix& (FitsImage::*)(),
		     CoordSystem, CoordFormat, char*);
  void getMinMax(const Vector&, Matrix& (FitsImage::*)());
  void getValue(const Vector&, Matrix& (FitsImage::*)());

  void printIntegerTcl(int);
  void printDoubleTcl(double, Precision);
  void printVectorTcl(const Vector&, Precision);
  void printCoordTcl(const Vector&, CoordSystem, CoordSystem, Precision);
  void coordToTclArray(const Vector&, CoordSystem, CoordSystem, Precision, 
		       const char*, const char*);
  void doubleToTclArray(double, Precision, const char*, const char*, 
			const char*, int);

  void centerImage();
  void reset();              // resets current position,zoom,rotation,flip
  void wcsAlign();           // sets wcs values

  void renderHighLite();     // render a hightlite bbox

  void renderCrosshair();    // render the crosshair
  void psCrosshair(PSColorSpace);
  void drawCrosshair();
  void eraseCrosshair();

  void graphCut(int, const Vector&);
  void graphCut(char*, char*, int, const Vector&);
  void fillArray(double*, double*, int, Vector&, int);
  void fillArrayMosaic(double*, double*, int, Vector&, int);
  void histogram(char*, char*);

  void unloadFits();         // unload all fits images
  double calcZoom(int,int,Vector);  // calculate the zoom factor
  void updateClip();         // set clip values
  void updateMinMax();       // calculate global min max
  void updateLowHigh();      // calculate global low high

  void deleteMarkers();             // delete all markers
  void psMarkers(int);              // generate ps for all markers
  void renderMarkers(const BBox&);  // render the markers into pixmap
  void renderMagnifierMarkers(const Matrix&, const BBox& bb);
  void unselectMarkers();           // unselect all markers
  void updateMarkers();             // update marker bbox
  void updateMarkerCoords(Matrix); // update marker coords

  void markerListHeader(ostream&, CoordSystem, const char*);
  void markerListCiaoHeader(ostream&, const char*);
  void markerListSAOtngHeader(ostream&, CoordSystem,
			      CoordFormat, const char*);

  void createMarker(Marker*);       // process new marker
  void parseMarker(MarkerFormat,istream&);

  void psLevel1Head(PSColorSpace, int, int);
  Vector psOrigin();

public:
  Frame(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame();

  int parse(istrstream&);

  double getZoom() {return zoom*wcsZoom;}
  double getMagnifierZoom() {return zoom*wcsZoom*magnifierZoom;}
  double getRotation() {return rotation+wcsRotation;}
  Vector getCenter();

  Matrix& getCanvasToRef() {return canvasToRef;}
  Matrix& getCanvasToWidget() {return canvasToWidget;}
  Matrix& getRefToCanvas() {return refToCanvas;}
  Matrix& getRefToWidget() {return refToWidget;}
  Matrix& getRefToWindow() {return refToWindow;}

  Vector map(const Vector&, CoordSystem, CoordSystem, int* =NULL);
  Vector mapWCS(const Vector&, CoordSystem, CoordSystem, int*);
  Vector mapImage(const Vector&, CoordSystem, int*);
  Vector mapPhysical(const Vector&, CoordSystem, int*);
  Vector mapAmplifier(const Vector&, CoordSystem, int*);
  Vector mapDetector(const Vector&, CoordSystem, int*);
  Vector mapRef(const Vector&, CoordSystem, int*);
  Vector mapUser(const Vector&, CoordSystem, int*);
  Vector mapCanvas(const Vector&, CoordSystem, int*);
  Vector mapPanner(const Vector&, CoordSystem, int*);

  double mapLenToRef(double, CoordSystem, CoordFormat =DEGREES);
  Vector mapLenToRef(const Vector&, CoordSystem, CoordFormat =DEGREES);

  double mapLenFromRef(double, CoordSystem, CoordFormat =DEGREES);
  Vector mapLenFromRef(const Vector&, CoordSystem, CoordFormat =DEGREES);

  double mapLenToRef(double, CoordFormat);
  Vector mapLenToRef(const Vector&, CoordFormat);

  double mapLenFromRef(double, CoordFormat);
  Vector mapLenFromRef(const Vector&, CoordFormat);

  double mapLenFromRef(const Vector&, const Vector&, CoordFormat);

  double wcsdist(const Vector&, const Vector&);

  void mapToWCSStr(const Vector&, CoordSystem, CoordSystem, CoordFormat, 
		   char*, int);
  void mapMatrixToWCSStr(const Vector&, CoordSystem, CoordFormat, char*, int,
			 Matrix& (FitsImage::*getMatrixData)(),
			 Matrix& (FitsImage::*getMatrixImage)());

  // Subcommand Functions

  void clearCmd();
  void colorScaleCmd(ColorScaleType);
  void getColormapCmd();
  void getColorScaleCmd();
  void getInfoCmd(char*);
  void getInfoCmd(const Vector&, CoordSystem, CoordSystem, CoordFormat, char*);
  void getNameCmd();
  void getValueCmd(const Vector&, CoordSystem);
  void highliteCmd(int);
  virtual void colormapCmd(int, float, float, int, unsigned short*, 
			   unsigned char*, int) =0;
  void colormapAreaCmd(int s, ColormapMode m) {cmapArea = s; cmapMode = m;}
  virtual void colormapBeginCmd() =0;
  virtual void colormapBeginCmd(const Vector&) =0;
  virtual void colormapMotionCmd(int, float, float, int, unsigned short*,
				 unsigned char*, int) =0;
  virtual void colormapEndCmd(int, float, float, int, unsigned short*,
			      unsigned char*, int) =0;
  void setNameCmd(char*);
  void warpCmd(const Vector&);

  // Clip Commands

  void clipMinMaxModeCmd(Frame::MinMaxMode);
  void clipMinMaxParamCmd(int);
  void clipModeCmd(ClipMode);
  void clipScopeCmd(ClipScope);
  void clipUserCmd(double, double);
  void clipUserLowCmd(double);
  void clipUserHighCmd(double);
  void clipZScaleParamCmd(float, int, int);
  void getClipCmd(Precision);
  void getClipCmd(const Vector&, CoordSystem, Precision);
  void getClipMinMaxModeCmd();
  void getClipMinMaxParamCmd() {printIntegerTcl(mmIncr);}
  void getClipModeCmd();
  void getClipScopeCmd();
  void getClipUserCmd(Precision);
  void getClipZScaleParamCmd();
  void getHistogramCmd(char*, char*);
  void getMinMaxCmd(const Vector&, CoordSystem);
  void hasDATAMINCmd();
  void hasIRAFMINCmd();

  // CrossHair Commands

  void crosshairCmd(int);
  void crosshairCmd(const Vector&, CoordSystem);
  void crosshairBeginCmd(const Vector&, CoordSystem);
  void crosshairMotionCmd(const Vector&, CoordSystem);
  void crosshairWarpCmd(const Vector&);
  void getCrosshairCmd(CoordSystem, Precision);
  void getCrosshairStatusCmd();
  void getHorzGraphCmd(char*, char*);
  void getHorzGraphCmd(char*, char*, const Vector&);
  void getVertGraphCmd(char*, char*);
  void getVertGraphCmd(char*, char*, const Vector&);

  // Fits Commands

  void getFitsCountCmd() {printIntegerTcl(fitsCount);}
  void getFitsHeaderCmd(int);
  void getFitsFileNameCmd(int,FileNameType);
  void getFitsFileNameCmd(const Vector&, CoordSystem,FileNameType);
  void getFitsFileNamesCmd(FileNameType);
  void getFitsObjectNameCmd(const Vector&, CoordSystem);
  void getFitsObjectNameCmd(int);
  void getFitsMasterCmd();
  void getFitsSizeCmd(CoordFormat);

  void loadFitsAllocCmd(const char*);
  void loadFitsChannelCmd(const char*, const char*);
  void loadFitsMMapCmd(const char* fn);
  void loadFitsShareCmd(ShmType, int, int, const char*);

  void loadArrAllocCmd(const char*);
  void loadArrChannelCmd(const char*, const char*);
  void loadArrMMapCmd(const char* fn);
  void loadArrShareCmd(ShmType, int, int, const char*);

  void loadMosaicAllocCmd(const char*);
  void loadMosaicChannelCmd(const char*, const char*);
  void loadMosaicMMapCmd(const char*);
  void loadMosaicShareCmd(ShmType, int, int, const char*);

  void loadMosaicExtMMapCmd(const char*);
  void loadMosaicExtShareCmd(ShmType, int, int, const char*);
  void loadMosaicExtAllocCmd(const char*);
  void loadMosaicExtChannelCmd(const char*, const char*);

  void loadFitsSlaveCmd(const char*, FitsFile*);

  void saveFitsFileCmd(const char*);
  void saveFitsChannelCmd(const char*);

  void updateFitsCmd() {updateNow(BASE);}
  void updateFitsCmd(int, BBox);
  void unloadFitsCmd();
  void scanOnLoadCmd(int);

  void hasFitsCmd();

  // Coordinate Commands

  void getCursorCmd(CoordSystem, CoordFormat);
  void getImageCoordCmd(const Vector& v, CoordSystem in, Precision p)
    {printCoordTcl(v, in, IMAGE, p);}
  void getPhysicalCoordCmd(const Vector& v, CoordSystem in, Precision p)
    {printCoordTcl(v, in, PHYSICAL, p);}
  void getAmplifierCoordCmd(const Vector& v, CoordSystem in, Precision p)
    {printCoordTcl(v, in, AMPLIFIER, p);}
  void getDetectorCoordCmd(const Vector& v, CoordSystem in, Precision p)
    {printCoordTcl(v, in, DETECTOR, p);}

  void getWCSCoordCmd(const Vector&, CoordSystem, CoordSystem, CoordFormat);
  void getWCSFlipCmd();
  void getWCSSystemCmd();
  void hasMosaicCmd();
  void hasAmplifierCmd();
  void hasDetectorCmd();
  void hasPhysicalCmd();
  void hasWCSCmd();
  void hasWCSEquCmd();
  void hasWCSLinearCmd();
  void mapCmd(const Vector&, CoordSystem, CoordSystem);

  // Marker Commands

  void createCircleCmd(const Vector&, double,
		       const char*, const char*, const char*, unsigned short);
  void createCircleCmd(const Vector&, CoordSystem, double, CoordFormat,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, double,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, CoordSystem, double, CoordFormat,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, double, double, int,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, CoordSystem, double, double, int,
		       CoordFormat,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, double*, int,
		       const char*, const char*, const char*, unsigned short);
  void createAnnulusCmd(const Vector&, CoordSystem, double*, int,
		       CoordFormat,
		       const char*, const char*, const char*, unsigned short);

  void createEllipseCmd(const Vector&, const Vector&, 
			double, const char*, const char*, 
			const char*, unsigned short);
  void createEllipseCmd(const Vector&, CoordSystem, const Vector&, 
			CoordFormat, double, const char*, const char*, 
			const char*, unsigned short);
  void createEllipseAnnulusCmd(const Vector&, const Vector&, 
			       const Vector&, int, double,
			       const char*, const char*, 
			       const char*, unsigned short);
  void createEllipseAnnulusCmd(const Vector&, CoordSystem, const Vector&, 
			       const Vector&, int, CoordFormat, double,
			       const char*, const char*, 
			       const char*, unsigned short);
  void createEllipseAnnulusCmd(const Vector& center,
			       double angle, int num,
			       Vector* radii,
			       const char* color, const char* font, 
			       const char* text, unsigned short prop);
  void createEllipseAnnulusCmd(const Vector& center, CoordSystem sys,
			       double angle, int num,
			       CoordFormat format, Vector* radii,
			       const char* color, const char* font, 
			       const char* text, unsigned short prop);

  void createBoxCmd(const Vector&, const Vector&, 
			  double, const char*, const char*, 
			  const char*, unsigned short);
  void createBoxCmd(const Vector&, CoordSystem, const Vector&, 
			  CoordFormat, double, const char*, const char*, 
			  const char*, unsigned short);
  void createBoxAnnulusCmd(const Vector&, const Vector&, 
			   const Vector&, int, double,
			   const char*, const char*, 
			   const char*, unsigned short);
  void createBoxAnnulusCmd(const Vector&, CoordSystem, const Vector&, 
			   const Vector&, int, CoordFormat, double,
			   const char*, const char*, 
			   const char*, unsigned short);
  void createBoxAnnulusCmd(const Vector& center,
			   double angle, int num,
			   Vector* size,
			   const char* color, const char* font, 
			   const char* text, unsigned short prop);
  void createBoxAnnulusCmd(const Vector& center, CoordSystem sys,
			   double angle, int num,
			   CoordFormat format, Vector* size,
			   const char* color, const char* font, 
			   const char* text, unsigned short prop);

  void createPolygonCmd(const Vector&, const Vector&,
			const char*, const char*, const char*, unsigned short);
  void createPolygonCmd(const Vector&, CoordSystem, const Vector&,
			const char*, const char*, const char*, unsigned short);
  void createPolygonCmd(List<Vertex>*,
			const char*, const char*, const char*, unsigned short);
  void createPolygonCmd(List<Vertex>*, CoordSystem,
			const char*, const char*, const char*, unsigned short);
  void createLineCmd(const Vector&, const Vector&, int, int,
		     const char*, const char*, const char*, unsigned short);
  void createLineCmd(const Vector&, const Vector&, CoordSystem, int, int,
		     const char*, const char*, const char*, unsigned short);
  void createTextCmd(const Vector&, const char*, const char*, 
		     const char*, unsigned short);
  void createTextCmd(const Vector&, CoordSystem, const char*, const char*, 
		     const char*, unsigned short);
  void createRulerCmd(const Vector&, const Vector&, CoordFormat,
		      const char*, const char*, const char*, unsigned short);
  void createRulerCmd(const Vector&, const Vector&, CoordSystem, CoordFormat,
		      const char*, const char*, const char*, unsigned short);
  void createCirclePointCmd(const Vector&,
			    const char*, const char*, const char*, 
			    unsigned short);
  void createCirclePointCmd(const Vector&, CoordSystem,
			    const char*, const char*, const char*, 
			    unsigned short);
  void createBoxPointCmd(const Vector&,
			 const char*, const char*, const char*, 
			 unsigned short);
  void createBoxPointCmd(const Vector&, CoordSystem,
			 const char*, const char*, const char*, 
			 unsigned short);
  void createDiamondPointCmd(const Vector&,
			     const char*, const char*, const char*, 
			     unsigned short);
  void createDiamondPointCmd(const Vector&, CoordSystem,
			     const char*, const char*, const char*, 
			     unsigned short);
  void createCrossPointCmd(const Vector&, 
			   const char*, const char*, const char*,
			   unsigned short);
  void createCrossPointCmd(const Vector&, CoordSystem, 
			   const char*, const char*, const char*,
			   unsigned short);
  void createExPointCmd(const Vector&,
			const char*, const char*, const char*, unsigned short);
  void createExPointCmd(const Vector&, CoordSystem, 
			const char*, const char*, const char*, unsigned short);
  void createArrowPointCmd(const Vector&, 
			   const char*, const char*, const char*, 
			   unsigned short);
  void createArrowPointCmd(const Vector&, CoordSystem, const char*, 
			   const char*, const char*, unsigned short);
  void createBoxCirclePointCmd(const Vector&,
			       const char*, const char*, const char*, 
			       unsigned short);
  void createBoxCirclePointCmd(const Vector&, CoordSystem,
			       const char*, const char*, const char*, 
			       unsigned short);

  void getMarkerAnnulusRadiusCmd(int, CoordFormat);
  void getMarkerBoxAngleCmd(int);
  void getMarkerBoxSizeCmd(int, CoordFormat);
  void getMarkerBoxAnnulusSizeCmd(int, CoordFormat);
  void getMarkerCenterCmd(int, CoordSystem);
  void getMarkerCenterCmd(int, CoordSystem, CoordFormat);
  void getMarkerCircleRadiusCmd(int, CoordFormat);
  void getMarkerColorCmd();
  void getMarkerColorCmd(int);
  void getMarkerEllipseAngleCmd(int);
  void getMarkerEllipseRadiusCmd(int, CoordFormat);
  void getMarkerEllipseAnnulusRadiusCmd(int, CoordFormat);
  void getMarkerFontCmd();
  void getMarkerFontCmd(int);
  void getMarkerHandleCmd(const Vector&);
  void getMarkerIdCmd(const Vector&);
  void getMarkerIdAllCmd();
  void getMarkerLineCmd(int, CoordSystem);
  void getMarkerLineCmd(int, CoordSystem, CoordFormat);
  void getMarkerLineArrowCmd(int);
  void getMarkerLineDistanceCmd(int, CoordFormat);
  void getMarkerPolygonAngleCmd(int);
  void getMarkerPolygonSegmentCmd(const Vector&);
  void getMarkerPropertyCmd(unsigned short);
  void getMarkerPropertyCmd(int, unsigned short);
  void getMarkerRulerDistanceCmd(int);
  void getMarkerRulerPointCmd(int, CoordSystem);
  void getMarkerRulerPointCmd(int, CoordSystem, CoordFormat);
  void getMarkerSelectedCmd();
  void getMarkerSelectedCmd(const Vector&);
  void getMarkerSelectedNumberCmd();
  void getMarkerTextCmd(int);
  void getMarkerTypeCmd(int);

  void markerAnnulusCreateRadiusCmd(int, const Vector&);
  void markerAnnulusDeleteRadiusCmd(int, int);
  void markerAnnulusRadiusCmd(int, double, double, int, CoordFormat);
  void markerAnnulusRadiusCmd(int, const char*, CoordFormat);
  void markerBackCmd();
  void markerBackCmd(int);
  void markerBoxAngleCmd(int, double);
  void markerBoxSizeCmd(int, const Vector&, CoordFormat);
  void markerBoxAnnulusSizeCmd(int, const Vector&, const Vector&, int, 
			       CoordFormat);
  void markerBoxAnnulusSizeCmd(int, const char*, CoordFormat);
  void markerBoxAnnulusCreateSizeCmd(int, const Vector&);
  void markerBoxAnnulusDeleteSizeCmd(int, int);
  void markerCallBackCmd(int, Marker::Callback, const char*, const char*);
  void markerCircleRadiusCmd(int, double, CoordFormat);
  void markerColorCmd(const char*);
  void markerColorCmd(int, const char*);
  void markerCommandCmd(MarkerFormat,const char*);
  void markerDeleteCmd();
  void markerDeleteCmd(int);
  void markerDeleteAllCmd();
  void markerDeleteLastCmd();
  void markerDeleteCallBackCmd(int, Marker::Callback, const char*);
  void markerEditBeginCmd(int, int);
  void markerEditBeginCmd(const Vector&, int);
  void markerEditMotionCmd(const Vector&, int h);
  void markerEditEndCmd();
  void markerEllipseRadiusCmd(int, const Vector&, CoordFormat);
  void markerEllipseAngleCmd(int, double);
  void markerEllipseAnnulusRadiusCmd(int, const Vector&, const Vector&, int, 
				     CoordFormat);
  void markerEllipseAnnulusRadiusCmd(int, const char*, CoordFormat);
  void markerEllipseAnnulusCreateRadiusCmd(int, const Vector&);
  void markerEllipseAnnulusDeleteRadiusCmd(int, int);
  void markerFontCmd(const char*);
  void markerFontCmd(int, const char*);
  void markerFrontCmd();
  void markerFrontCmd(int);
  void markerLineCmd(int, const Vector&, const Vector&, CoordSystem);
  void markerLineArrowCmd(int, int, int);
  void markerListCmd(MarkerFormat, CoordSystem, CoordFormat, Delim);
  void markerListCmd(MarkerFormat, CoordSystem, CoordFormat, 
		     unsigned short, int, Delim);
  void markerLoadCmd(MarkerFormat,const char*);
  void markerLoadCmd(MarkerFormat,int);
  void markerLoadFitsCmd(const char*);
  void markerMoveCmd(const Vector&);
  void markerMoveCmd(int id, const Vector&);
  void markerMoveBeginCmd(const Vector&);
  void markerMoveBeginCmd(int, const Vector&);
  void markerMoveMotionCmd(const Vector&);
  void markerMoveMotionCmd(int, const Vector&);
  void markerMoveEndCmd();
  void markerMoveEndCmd(int);
  void markerMoveToCmd(const Vector&, CoordSystem);
  void markerMoveToCmd(int, const Vector&, CoordSystem);
  void markerPolygonAngleCmd(int, double);
  void markerPolygonCreateVertexCmd(int, int, const Vector&);
  void markerPolygonDeleteVertexCmd(int, int);
  void markerPropertyCmd(unsigned short, int);
  void markerPropertyCmd(int, unsigned short, int);
  void markerRotateBeginCmd(int);
  void markerRotateBeginCmd(const Vector&);
  void markerRotateMotionCmd(const Vector&, int h);
  void markerRotateEndCmd();
  void markerRulerPointCmd(int, const Vector&, const Vector&, CoordSystem);
  void markerRulerSystemCmd(int, CoordSystem, CoordFormat);
  void markerSaveCmd(const char*, MarkerFormat, CoordSystem, CoordFormat);
  void markerSelectCmd(int);
  void markerSelectCmd(const Vector&);
  void markerSelectAllCmd();
  void markerSelectToggleCmd(const Vector&);
  void markerTextCmd(int, const char*);
  void markerUnselectCmd(int);
  void markerUnselectAllCmd();

  void regionSelectBeginCmd(const Vector&);
  void regionSelectMotionCmd(const Vector&);
  void regionSelectEndCmd();
  void regionSelectShiftEndCmd();

  // Pan Zoom Rotate Orient Commands

  void centerCmd();
  void getPanCmd(CoordSystem, Precision);
  void getOrientCmd();
  void getRotateCmd(Precision);
  void getWCSAlignCmd();
  void getWCSZoomCmd(Precision);
  void getZoomCmd(Precision);
  void orientCmd(Orientation);
  void panCmd(const Vector&, const Vector&);
  void panCmd(const Vector&, CoordSystem);
  void panBBoxCmd(BBox);
  void panMotionBeginCmd(const Vector&);
  void panMotionCmd(const Vector&);
  void panMotionEndCmd(const Vector&);
  void panToCmd(const Vector&, CoordSystem);
  void rotateCmd(double);
  void rotateToCmd(double);
  void rotateMotionBeginCmd();
  void rotateMotionCmd(double);
  void rotateMotionEndCmd();
  void wcsAlignCmd(int);
  void wcsZoomCmd(double);
  void zoomCmd(double);
  void zoomAboutCmd(double, const Vector&, CoordSystem);
  void zoomToCmd(double);
  void zoomToAboutCmd(double, const Vector&, CoordSystem);
  void zoomToFitCmd();

  // Panner/Magnifier Commands

  void magnifierCmd(char*, int, int);
  void magnifierCmd(int);
  void magnifierGraphicsCmd(int);
  void magnifierCursorCmd(int);
  void magnifierModeCmd(MagnifierMode);
  void magnifierZoomCmd(double);

  void pannerCmd(char*, int, int);
  void pannerCmd(int);
  void updatePannerCmd();
  void updateMagnifierCmd(const Vector& v) {updateMagnifier(v);}

  // Postscript Commands
       
  void psInterpolateCmd(int w) {psInterpolate = w;}
  void psLevelCmd(int l) {psLevel = (l == 2) ? 2 : 1;}
  void psResolutionCmd(int r) {psResolution = r;}
  void psColorSpaceCmd(PSColorSpace c) {psColorSpace = c;}

  // Bin Commands

  void binCmd(double, const Vector&, CoordSystem, 
	      const char*, const char*, const char*);
  void binCmd(double, const char*, const char*, const char*);
  void binAboutCmd(const Vector&, CoordSystem);
  void binColsCmd(const char*, const char*);
  void binFactorCmd(double);
  void binFactorAboutCmd(double, const Vector&, CoordSystem);
  void binFactorToCmd(double);
  void binFactorToAboutCmd(double, const Vector&, CoordSystem);
  void binToFitCmd();
  void binFunctionCmd(FitsHist::Function);
  void binBufferSizeCmd(int);
  void binFilterCmd(const char*);
  void getBinFactorCmd(Precision);
  void getBinFunctionCmd();
  void getBinBufferSizeCmd() {printIntegerTcl(binBufferSize);}
  void getBinCursorCmd(Precision);
  void getBinFilterCmd();
  void getBinColsCmd();
  void getBinListCmd();
  void hasBinCmd();

  // Contour Commands

  void contourCreateCmd(const char*, int, float, ContourMethod);
  void contourCreateCmd(const char*, int, float, ContourMethod,
			ColorScaleType);
  void contourCreateCmd(const char*, int, float, ContourMethod,
			float, float, ColorScaleType);
  void contourCreateCmd(const char*, float, ContourMethod, const char*);
  void contourLoadCmd(const char*, const char*, CoordSystem);
  void contourCopyCmd(CoordSystem);
  void contourDeleteCmd();
  void contourPasteCmd(const char*, void*, CoordSystem);
  void contourSaveCmd(const char*, CoordSystem);
  void getContourLevelCmd(int, float, float, ColorScaleType);
  void getContourMethodCmd();
  void hasContourCmd();

  // Grid Commands

  void gridCmd(CoordSystem, const char*, int);
  void gridDeleteCmd();
  void hasGridCmd();
  void getGridSystemCmd();
  void getGridOptionCmd();

  // IIS Commands

  void iisCmd(int, int);
  void iisEraseCmd();
  void iisMessageCmd(const char*);
  void iisSetCmd(const char*, int, int, int, int);
  void iisWCSCmd(const Matrix&, const Vector&, int);
  void iisGetCmd(char*, int, int, int, int);
  void iisGetWidthCmd();
  void iisGetHeightCmd();
  void iisUpdateCmd() {updateNow(MATRIX);}
  void iisSetCursorCmd(const Vector&, CoordSystem);
  void iisGetCursorCmd();
  void iisCursorModeCmd(int);
  void hasIISCmd();

  // Pixel Table

  void getPixelTableCmd(const Vector&, CoordSystem, int, char*);
  void getPixelTable(const Vector&, CoordSystem,
			  Matrix& (FitsImage::*matrixToData)(),
			  int, char*);
};

#endif
