#ifndef __marker_h__
#define __marker_h__

#include <iostream.h>
#include <strstream.h>
#include <iomanip.h>

#include <tk.h>

#include "vector.h"
#include "callback.h"
#include "list.h"

class Frame;

static int sequenceID = 1;

class Marker {
public:
  enum Property {NONE=0, SELECT=1, EDIT=2, MOVE=4, ROTATE=8, DELETE=16, 
		 INCLUDE=32, FIXED=64};
  // Select-- user may select the marker
  // Edit-- user may edit the marker
  // Move-- user may move the marker
  // Rotate-- user may rotate the marker
  // Delete-- user may delete the marker
  // Include-- marker is 'included' or 'excluded' ie '+' or '-'
  // Fixed-- marker is fixed in size (not scaled based on zoom)

  enum Callback {SELECTCB, EDITCB, MOVECB, ROTATECB, DELETECB, 
		 TEXTCB, COLORCB, PROPERTYCB, FONTCB};

protected:
  int id;
  char type[16];
  Vector center;                // image coordinates
  BBox bbox;                    // canvas coordinates
  BBox allBBox;                 // canvas coordinates

  Vector* handle;              // canvas coordinates 
  int numHandle;

  int color;
  char* colorName;
  unsigned short properties;
  int selected;

  char* text;
  Tk_Font font;              // Tk font

  Display* display;
  GC gc;
  GC gcxor;

  List<CallBack> selectCB;
  List<CallBack> moveCB;
  List<CallBack> editCB;
  List<CallBack> rotateCB;
  List<CallBack> deleteCB;
  List<CallBack> textCB;
  List<CallBack> colorCB;
  List<CallBack> propertyCB;
  List<CallBack> fontCB;

  Frame* parent;
  Marker* previous_;
  Marker* next_;

protected:
  Vector* arrow(const Vector&, const Vector&);
  void renderArrow(Drawable, const Vector&, const Vector&, GC);
  void psArrow(const Vector&, const Vector&);

  void includeExclude();
  virtual void dash(GC);
  void noDash(GC);
  void psIncludeExclude();
  virtual void psDash();
  void psNoDash();

  void doCallBacks(List<CallBack>*);
  void psColor(int);
  void calcAllBBox();
  void drawHandles();
  void drawText();
  virtual void listProperties(ostream&);
  void listCoordSystem(ostream&, int);
  void listProsCoordSystem(ostream&, int);

public:
  Marker(Frame*, const Vector&, const char*, const char*, const char*, unsigned short);
  virtual ~Marker();

  virtual void updateBBox() =0;
  virtual void updateCoords(Matrix&);
  virtual void draw();          // updates bbox and renders to pixmap
  virtual void drawXOR() =0;    // updates bbox and renders to window 
  virtual void drawMagnifier(const Matrix&) =0;
  virtual void move(const Vector& v);
  virtual void moveTo(const Vector& v);
  virtual void editBegin(int) {}
  virtual void edit(const Vector& v, int h) {}
  virtual void editEnd() {}
  virtual void rotate(const Vector& v, int h) {}
  virtual void ps(int);
  virtual int isIn(const Vector& v) {return bbox.isIn(v);}

  virtual int isPolygon() {return 0;}

  virtual void list(ostream&, int, int, const char*);
  virtual void listCiao(ostream&, int, int, const char*) {}
  virtual void listFtools(ostream&, int, int, const char*) {}
  virtual void listSAOtng(ostream&, int, int, const char*) {}
  virtual void listSAOimage(ostream&, const char*) {}
  virtual void listPros(ostream&, int, int, const char*) {}

  const BBox& getBBox() {return bbox;}
  const BBox& getAllBBox() {return allBBox;}
  int isVisible(const BBox&);

  void select();
  void unselect();
  void toggleSelect();
  int isSelected() {return selected;}

  void setColor(const char*);
  const char* getColorName() {return colorName;}

  void setText(const char*);
  const char* getText() {return text;}
  void setFont(const char*);
  const char* getFont();

  int onHandle(const Vector& v);

  int getId() {return id;}
  const char* getType() {return type;}
  Vector getCenter() {return center;}

  int canSelect() {return (properties & SELECT) ? 1:0;}
  int canEdit() {return (properties & EDIT) ? 1:0;}
  int canMove() {return (properties & MOVE) ? 1:0;}
  int canRotate() {return (properties & ROTATE) ? 1:0;}
  int canDelete() {return (properties & DELETE) ? 1:0;}
  int isFixed() {return (properties & FIXED) ? 1:0;}
  void setProperty(unsigned short, int);
  int getProperty(unsigned short);

  int setCallBack(Callback, const char*, const char*);
  int deleteCallBack(Callback, const char*);

  Marker* previous() {return previous_;}
  void setPrevious(Marker* m) {previous_ = m;}
  Marker* next() {return next_;}
  void setNext(Marker* m) {next_ = m;}
};

#endif
