#include <iostream.h>
#include <strstream.h>
#include <iomanip.h>

#include "../../tk8.3.2/generic/tkCanvas.h"

#include "ps.h"
#include "frame.h"
#include "framepseudo8.h"
#include "frametrue.h"
#include "fitsimage.h"
#include "contour.h"
#include "grid.h"
#include "util.h"
#include "nan.h"

// this is copied from tkCanvPs.c

typedef struct TkPostscriptInfo {
    int x, y, width, height;	/* Area to print, in canvas pixel
				 * coordinates. */
    int x2, y2;			/* x+width and y+height. */
    char *pageXString;		/* String value of "-pagex" option or NULL. */
    char *pageYString;		/* String value of "-pagey" option or NULL. */
    double pageX, pageY;	/* Postscript coordinates (in points)
				 * corresponding to pageXString and
				 * pageYString. Don't forget that y-values
				 * grow upwards for Postscript! */
    char *pageWidthString;	/* Printed width of output. */
    char *pageHeightString;	/* Printed height of output. */
    double scale;		/* Scale factor for conversion: each pixel
				 * maps into this many points. */
    Tk_Anchor pageAnchor;	/* How to anchor bbox on Postscript page. */
    int rotate;			/* Non-zero means output should be rotated
				 * on page (landscape mode). */
    char *fontVar;		/* If non-NULL, gives name of global variable
				 * containing font mapping information.
				 * Malloc'ed. */
    char *colorVar;		/* If non-NULL, give name of global variable
				 * containing color mapping information.
				 * Malloc'ed. */
    char *colorMode;		/* Mode for handling colors:  "monochrome",
				 * "gray", or "color".  Malloc'ed. */
    int colorLevel;		/* Numeric value corresponding to colorMode:
				 * 0 for mono, 1 for gray, 2 for color. */
    char *fileName;		/* Name of file in which to write Postscript;
				 * NULL means return Postscript info as
				 * result. Malloc'ed. */
    char *channelName;		/* If -channel is specified, the name of
                                 * the channel to use. */
    Tcl_Channel chan;		/* Open channel corresponding to fileName. */
    Tcl_HashTable fontTable;	/* Hash table containing names of all font
				 * families used in output.  The hash table
				 * values are not used. */
    int prepass;		/* Non-zero means that we're currently in
				 * the pre-pass that collects font information,
				 * so the Postscript generated isn't
				 * relevant. */
    int prolog;			/* Non-zero means output should contain
				   the file prolog.ps in the header. */
} TkPostscriptInfo;

int Frame::postscriptProc(int prepass)
{
  TkPostscriptInfo* psInfo = (TkPostscriptInfo*)(((TkCanvas*)canvas)->psInfo);

  if (!visible)
    return TCL_OK;

  if (prepass)
    return TCL_OK;

  char buf[256];
  Vector org = psOrigin();

  // Image

  if (fits) {

    // save graphics state

    Tcl_AppendResult(interp, "gsave\n", NULL);

    float scale = psInfo->scale * psResolution / 72.;
    int width = (int)(options->width * scale);
    int height = (int)(options->height * scale);

    // scale

    {
      ostrstream str(buf,256);
      str << org[0] << ' ' << org[1] << " translate" << endl
	  << 1/scale << ' ' << 1/scale << " scale" << endl
	  << ends;
      Tcl_AppendResult(interp, buf, NULL);
    }

    switch (psLevel) {
    case 1:
      switch (psColorSpace) {
      case BW:
      case GRAY:
	psLevel1(GRAY, width, height, scale);
	break;
      case RGB:
      case CMYK:
	psLevel1(RGB, width, height, scale);
	break;
      }
      break;
    case 2:
      psLevel2(psColorSpace, width, height, scale);
      break;
    }

    // restore gstate

    Tcl_AppendResult(interp, "grestore\n", NULL);
  }

  // Markers & Contours & Grids

  // clip path
  // I don't know why the height needs to be reduced by one, however, it works

  {
    ostrstream str(buf,256);
    str << org[0] << ' ' << org[1] << ' ' 
	<< options->width << ' ' << options->height-1
	<< " rectclip" << endl << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  switch (psLevel) {
  case 1:
    switch (psColorSpace) {
    case BW:
    case GRAY:
      if (contour)
	contour->ps(GRAY);
      if (grid)
	grid->ps(GRAY);
      psMarkers(GRAY);
      if (useCrosshair)
	psCrosshair(GRAY);
      break;
    case RGB:
    case CMYK:
      if (contour)
	contour->ps(RGB);
      if (grid)
	grid->ps(RGB);
      psMarkers(RGB);
      if (useCrosshair)
	psCrosshair(RGB);
      break;
    }
    break;
  case 2:
    if (contour)
      contour->ps(psColorSpace);
    if (grid)
      grid->ps(psColorSpace);
    psMarkers(psColorSpace);
      if (useCrosshair)
	psCrosshair(psColorSpace);
    break;
  }

  return TCL_OK;
}

void Frame::psLevel1Head(PSColorSpace mode, int width, int height)
{
  char buf[512];
  ostrstream str(buf,512);

  switch (mode) {
  case BW:
  case GRAY:
    str << "/picstr " << dec << width << " string def" << endl
	<< width << ' ' << height << " 8 matrix" << endl
	<< "{currentfile picstr readhexstring pop}" << endl
	<< "image" << endl;
    break;
  case RGB:
  case CMYK:
    str << "/picstr " << dec << width*3 << " string def" << endl
	<< width << ' ' << height << " 8 matrix" << endl
	<< "{currentfile picstr readhexstring pop}" << endl
	<< "false 3 colorimage" << endl;
    break;
  }

  str << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

void Frame::psLevel1(PSColorSpace mode, int width, int height, float scale)
{
  psLevel1Head(mode, width, height);

  NoCompressAsciiHex filter;

  // size = 2 char per byte * 3 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*3*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix mm = psMatrix(scale, width, height);

  for (int j=0; j<height; j++) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;
	    
    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * mm).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else
	  current = fits;
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void FrameTrueColor::psLevel2Head(PSColorSpace mode, int width, int height)
{
  char buf[4096];
  ostrstream str(buf,4096);

  switch (mode) {
  case BW:
  case GRAY:
    str << "/DeviceGray setcolorspace" << endl;
    break;
  case RGB:
    str << "/DeviceRGB setcolorspace" << endl;
    break;
  case CMYK:
    str << "/DeviceCMYK setcolorspace" << endl;
    break;
  }

  str << "<<" << endl
      << "/ImageType 1" << endl
      << "/Width " << dec << width << endl
      << "/Height " << dec << height << endl
      << "/BitsPerComponent 8" << endl;

  switch (mode) {
  case BW:
  case GRAY:
    str << "/Decode [0 1]" << endl;
    break;
  case RGB:
    str << "/Decode [0 1 0 1 0 1]" << endl;
    break;
  case CMYK:
    str << "/Decode [0 1 0 1 0 1 0 1]" << endl;
    break;
  }

  if (psInterpolate)
    str << "/Interpolate true" << endl;

  str << "/ImageMatrix matrix" << endl
      << "/DataSource currentfile" << endl
      << "/ASCII85Decode filter" << endl
      << "/RunLengthDecode filter" << endl
      << ">>" << endl
      << "image" << endl
      << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

void FrameTrueColor::psLevel2(PSColorSpace mode, int width, int height,
			      float scale)
{
  psLevel2Head(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte * 4 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*4*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix mm = psMatrix(scale, width, height);

  for (int j=0; j<height; j++) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;

    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * mm).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else
	  current = fits;
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void FramePseudoColor8::psLevel2Head(PSColorSpace mode, int width, int height)
{
  char buf[4096];
  ostrstream str(buf,4096);
  const unsigned short white = 255;

  switch (mode) {
  case BW:
  case GRAY:
    {
      str << "[/Indexed /DeviceGray 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned short value = 0;

	unsigned char red;
	unsigned char green;
	unsigned char blue;
	if (psLevel2BaseColor(i, &red, &green, &blue))
	  value = RGB2Gray(red, green, blue);
	else
	  for (int j=0; j<colorCount; j++)
	    if (indexCells[j]==i) {
	      value = RGB2Gray(colorCells[j*3+2], colorCells[j*3+1], 
			       colorCells[j*3]);
	      break;
	    }

	str << hex << setfill('0') << setw(2) << value << ' ';

	if (++count == 16) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << white << endl;

    }
    break;
  case RGB:
    {
      str << "[/Indexed /DeviceRGB 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned short red = 0;
	unsigned short green = 0;
	unsigned short blue = 0;

	unsigned char r;
	unsigned char g;
	unsigned char b;
	if (psLevel2BaseColor(i, &r, &g, &b)) {
	  red = r;
	  green = g;
	  blue = b;
	}
	else
	  for (int j=0; j<colorCount; j++)
	    if (indexCells[j]==i) {
	      red = colorCells[j*3+2];
	      green = colorCells[j*3+1];
	      blue = colorCells[j*3];
	      break;
	    }

	str << hex << setfill('0') << setw(2) << red 
	    << hex << setfill('0') << setw(2) << green
	    << hex << setfill('0') << setw(2) << blue
	    << ' ';

	if (++count == 8) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << white
	  << hex << setfill('0') << setw(2) << white
	  << hex << setfill('0') << setw(2) << white
	  << ' ' << endl;

    }
    break;
  case CMYK:
    {
      str << "[/Indexed /DeviceCMYK 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned char c = 0;
	unsigned char m = 0;
	unsigned char y = 0;
	unsigned char b = 255;

	unsigned char red;
	unsigned char green;
	unsigned char blue;
	if (psLevel2BaseColor(i, &red, &green, &blue))
	  RGB2CMYK(red, green, blue, &c, &m, &y, &b);
	else
	  for (int j=0; j<colorCount; j++)
	    if (indexCells[j]==i) {
	      RGB2CMYK(colorCells[j*3+2], colorCells[j*3+1], colorCells[j*3],
		       &c, &m, &y, &b);
	      break;
	    }

	unsigned short cyan = c;
	unsigned short magenta = m;
	unsigned short yellow = y;
	unsigned short black = b;

	str << hex << setfill('0') << setw(2) << cyan 
	    << hex << setfill('0') << setw(2) << magenta
	    << hex << setfill('0') << setw(2) << yellow
	    << hex << setfill('0') << setw(2) << black
	    << ' ';

	if (++count == 8) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << ' ' << endl;
    }
    break;
  }

  str << '>' << endl
      << "] setcolorspace" << endl
      << "<<" << endl
      << "/ImageType 1" << endl
      << "/Width " << dec << width << endl
      << "/Height " << dec << height << endl
      << "/BitsPerComponent 8" << endl
      << "/Decode [0 255]" << endl;

  if (psInterpolate)
    str << "/Interpolate true" << endl;

  str << "/ImageMatrix matrix" << endl
      << "/DataSource currentfile" << endl
      << "/ASCII85Decode filter" << endl
      << "/RunLengthDecode filter" << endl
      << ">>" << endl
      << "image" << endl
      << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

int FramePseudoColor8::psLevel2BaseColor(int i, unsigned char* red, 
				  unsigned char* green, unsigned char* blue)
{
  if (i==blackColor->pixel) {
    *blue = 0;
    *green = 0;
    *red = 0;
  }
  else if (i==whiteColor->pixel) {
    *blue = 255;
    *green = 255;
    *red = 255;
  }
  else if (i==redColor->pixel) {
    *blue = 0;
    *green = 0;
    *red = 255;
  }
  else if (i==greenColor->pixel) {
    *blue = 0;
    *green = 255;
    *red = 0;
  }
  else if (i==blueColor->pixel) {
    *blue = 255;
    *green = 0;
    *red = 0;
  }
  else if (i==cyanColor->pixel) {
    *blue = 255;
    *green = 255;
    *red = 0;
  }
  else if (i==magentaColor->pixel) {
    *blue = 255;
    *green = 0;
    *red = 255;
  }
  else if (i==yellowColor->pixel) {
    *blue = 0;
    *green = 255;
    *red = 255;
  }
  else
    return 0;

  return 1;
}

void FramePseudoColor8::psLevel2(PSColorSpace mode, int width, int height, 
			  float scale)
{
  psLevel2Head(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte * 3 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*3*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned short* table = colorScale->psIndex();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix mm = psMatrix(scale, width, height);

  for (int j=0; j<height; j++) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * mm).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll)
	    filter << table[0];
	  else if (value >= hh)
	    filter << table[length];
	  else
	    filter << table[(int)(((value - ll)/diff * length) + .5)];

	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else {
	  filter << 255;
	  current = fits;
	}
      }

      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

Matrix Frame::psMatrix(float psZoom, int width, int height)
{
  Matrix m;
  if (!isIIS()) {
    Vector center = getCenter();
    m = Translate(-center) *
      FlipY() *
      Translate(center);
  }

  return m *
    Translate(-cursor) *                // translate to cursor position
    Rotate(rotation) *                  // rotate about cursor position
    Rotate(wcsRotation) *               // rotate about center position
    orientationMatrix *                 // flip x/y axis about cursor position
    wcsOrientationMatrix *              // flip x/y axis about center
    Scale(zoom) *                       // scale about cursor position
    Scale(wcsZoom) *                    // scale about wcs scale
    Scale(psZoom) *

    FlipY() *
    Translate(Vector(width,height)/2.);
}

Vector Frame::psOrigin()
{
  // compute coordinates of lower-left corner of the image
  // taking into account the anchor position for the image.
  // this mapping is different than in widget.c, due to postscript origin 
  // (y is flipped)

  double xx = options->x;
  double yy = Tk_CanvasPsY(canvas, options->y);
     
  switch (options->anchor) {
  case TK_ANCHOR_NW:
    yy -= options->height;
    break;
  case TK_ANCHOR_N:
    xx -= options->width/2.0;
    yy -= options->height;
    break;
  case TK_ANCHOR_NE:
    xx -= options->width;
    yy -= options->height;
    break;
  case TK_ANCHOR_E:
    xx -= options->width;
    yy -= options->height/2.0;
    break;
  case TK_ANCHOR_SE:
    xx -= options->width;
    break;
  case TK_ANCHOR_S:
    xx -= options->width/2.0;
    break;
  case TK_ANCHOR_SW:
    break;
  case TK_ANCHOR_W:
    yy -= options->height/2.0;
    break;
  case TK_ANCHOR_CENTER:
    xx -= options->width/2.0;
    yy -= options->height/2.0;
    break;
  }

  return Vector(xx,yy);
}
